@@ -19,7 +19,7 @@
 
 """The Translate Toolkit is a Python package that assists in localization of software.
 
-See U{http://translate.sourceforge.net} or U{http://translate.org.za} for more information.
+See U{http://translate.sourceforge.net/wiki/toolkit/index} or U{http://translate.org.za} for more information.
 
 @organization: Zuza Software Foundation
 @copyright: 2002-2008 Zuza Software Foundation
