@@ -105,6 +105,8 @@ class oo2po:
     def convertstore(self, theoofile, duplicatestyle="msgctxt"):
         """converts an entire oo file to a base class format (.po or XLIFF)"""
         thetargetfile = po.pofile()
+        thetargetfile.setsourcelanguage(self.sourcelanguage)
+        thetargetfile.settargetlanguage(self.targetlanguage)
         # create a header for the file
         bug_url = 'http://qa.openoffice.org/issues/enter_bug.cgi' + ('''?subcomponent=ui&comment=&short_desc=Localization issue in file: %(filename)s&component=l10n&form_name=enter_issue''' % {"filename": theoofile.filename}).replace(" ", "%20").replace(":", "%3A")
         targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit", x_accelerator_marker="~", report_msgid_bugs_to=bug_url)
@@ -151,7 +153,7 @@ def convertoo(inputfile, outputfile, templates, pot=False, sourcelanguage=None,
 
 def main(argv=None):
     from translate.convert import convert
-    formats = {"oo":("po",convertoo), "sdf":("po",convertoo)}
+    formats = {"oo":("po", convertoo), "sdf":("po", convertoo)}
     # always treat the input as an archive unless it is a directory
     archiveformats = {(None, "input"): oo.oomultifile}
     parser = convert.ArchiveConvertOptionParser(formats, usepots=True, description=__doc__, archiveformats=archiveformats)
