@@ -105,6 +105,8 @@ class oo2xliff:
     def convertstore(self, theoofile, duplicatestyle="msgctxt"):
         """converts an entire oo file to a base class format (.po or XLIFF)"""
         thetargetfile = xliff.xlifffile()
+        thetargetfile.setsourcelanguage(self.sourcelanguage)
+        thetargetfile.settargetlanguage(self.targetlanguage)
         # create a header for the file
         bug_url = 'http://qa.openoffice.org/issues/enter_bug.cgi' + ('''?subcomponent=ui&comment=&short_desc=Localization issue in file: %(filename)s&component=l10n&form_name=enter_issue''' % {"filename": theoofile.filename}).replace(" ", "%20").replace(":", "%3A")
         # go through the oo and convert each element
@@ -123,7 +125,10 @@ def convertoo(inputfile, outputfile, templates, pot=False, sourcelanguage=None,
     """reads in stdin using inputstore class, converts using convertorclass, writes to stdout"""
     inputstore = oo.oofile()
     if hasattr(inputfile, "filename"):
-        inputstore.filename = inputfile.filename
+        inputfilename = inputfile.filename
+    else:
+        inputfilename = "(input file name not known)"
+    inputstore.filename = inputfilename
     inputstore.parse(inputfile.read())
     if not sourcelanguage:
         testlangtype = targetlanguage or (inputstore and inputstore.languages[0]) or ""
@@ -132,9 +137,9 @@ def convertoo(inputfile, outputfile, templates, pot=False, sourcelanguage=None,
         else:
             sourcelanguage = "en-US"
     if not sourcelanguage in inputstore.languages:
-        print >> sys.stderr, "Warning: sourcelanguage %s not found in inputfile (contains %s)" % (sourcelanguage, ", ".join(inputstore.languages))
+        print >> sys.stderr, "Warning: sourcelanguage '%s' not found in inputfile '%s' (contains %s)" % (sourcelanguage, inputfilename, ", ".join(inputstore.languages))
     if not pot and targetlanguage and targetlanguage not in inputstore.languages:
-        print >> sys.stderr, "Warning: targetlanguage %s not found in inputfile (contains %s)" % (targetlanguage, ", ".join(inputstore.languages))
+        print >> sys.stderr, "Warning: targetlanguage '%s' not found in inputfile '%s' (contains %s)" % (targetlanguage, inputfilename, ", ".join(inputstore.languages))
     convertor = oo2xliff(sourcelanguage, targetlanguage, blankmsgstr=pot, long_keys=multifilestyle!="single")
     outputstore = convertor.convertstore(inputstore, duplicatestyle)
     if outputstore.isempty():
@@ -144,7 +149,7 @@ def convertoo(inputfile, outputfile, templates, pot=False, sourcelanguage=None,
 
 def main(argv=None):
     from translate.convert import convert
-    formats = {"oo":("xlf",convertoo), "sdf":("xlf",convertoo)}
+    formats = {"oo":("xlf", convertoo), "sdf":("xlf", convertoo)}
     # always treat the input as an archive unless it is a directory
     archiveformats = {(None, "input"): oo.oomultifile}
     parser = convert.ArchiveConvertOptionParser(formats, usepots=False, description=__doc__, archiveformats=archiveformats)
