@@ -3,8 +3,6 @@
 from translate.convert import po2html
 from translate.convert import test_convert
 from translate.misc import wStringIO
-from translate.storage import po
-from translate.storage import html
 
 class TestPO2Html:
     def converthtml(self, posource, htmltemplate):
@@ -88,7 +86,7 @@ class TestPO2HtmlCommand(test_convert.TestConvertCommand, TestPO2Html):
     def test_help(self):
         """tests getting help"""
         options = test_convert.TestConvertCommand.test_help(self)
-        options = self.help_check(options, "-tTEMPLATE, --template=TEMPLATE")
-        options = self.help_check(options, "-wWRAP, --wrap=WRAP")
+        options = self.help_check(options, "-t TEMPLATE, --template=TEMPLATE")
+        options = self.help_check(options, "-w WRAP, --wrap=WRAP")
         options = self.help_check(options, "--fuzzy")
         options = self.help_check(options, "--nofuzzy", last=True)
