@@ -1,12 +1,10 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 
 from translate.convert import po2prop
-from translate.convert import prop2po
 from translate.convert import test_convert
 from translate.misc import wStringIO
 from translate.storage import po
-from translate.storage import properties
-from py import test
 
 class TestPO2Prop:
     def po2prop(self, posource):
@@ -14,7 +12,7 @@ class TestPO2Prop:
         inputfile = wStringIO.StringIO(posource)
         inputpo = po.pofile(inputfile)
         convertor = po2prop.po2prop()
-        outputprop = convertor.convertfile(inputpo)
+        outputprop = convertor.convertstore(inputpo)
         return outputprop
 
     def merge2prop(self, propsource, posource, personality="java"):
@@ -24,7 +22,7 @@ class TestPO2Prop:
         templatefile = wStringIO.StringIO(propsource)
         #templateprop = properties.propfile(templatefile)
         convertor = po2prop.reprop(templatefile)
-        outputprop = convertor.convertfile(inputpo, personality=personality)
+        outputprop = convertor.convertstore(inputpo, personality=personality)
         print outputprop
         return outputprop
 
@@ -107,7 +105,7 @@ class TestPO2PropCommand(test_convert.TestConvertCommand, TestPO2Prop):
     def test_help(self):
         """tests getting help"""
         options = test_convert.TestConvertCommand.test_help(self)
-        options = self.help_check(options, "-tTEMPLATE, --template=TEMPLATE")
+        options = self.help_check(options, "-t TEMPLATE, --template=TEMPLATE")
         options = self.help_check(options, "--fuzzy")
         options = self.help_check(options, "--personality=TYPE")
         options = self.help_check(options, "--nofuzzy", last=True)
