@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # 
-# Copyright 2004-2007 Zuza Software Foundation
+# Copyright 2004-2008 Zuza Software Foundation
 # 
 # This file is part of translate.
 #
@@ -43,13 +43,18 @@ try:
     from translate.storage import xliff
 except ImportError, e:
     xliff = None
+# The import of xliff fail silently in the absence of lxml if another module
+# already tried to import it unsuccessfully, so let's make 100% sure:
+if not hasattr(xliff, "xliffunit"):
+    xliff = None
 import re
 
 # These are some regular expressions that are compiled for use in some tests
 
 # printf syntax based on http://en.wikipedia.org/wiki/Printf which doens't cover everything we leave \w instead of specifying the exact letters as
 # this should capture printf types defined in other platforms.
-printf_pat = re.compile('%((?:(?P<ord>\d+)\$)*(?P<fullvar>[+#-]*(?:\d+)*(?:\.\d+)*(hh\|h\|l\|ll)*(?P<type>[\w%])))')
+# extended to support Python named format specifiers
+printf_pat = re.compile('%((?:(?P<ord>\d+)\$|\((?P<key>\w+)\))?(?P<fullvar>[+#-]*(?:\d+)?(?:\.\d+)?(hh\|h\|l\|ll)?(?P<type>[\w%])))')
 
 # The name of the XML tag
 tagname_re = re.compile("<[\s]*([\w\/]*)")
@@ -110,13 +115,12 @@ class FilterFailure(Exception):
     def __init__(self, messages):
         if not isinstance(messages, list):
             messages = [messages]
-        strmessages = []
-        for message in messages:
-            if isinstance(message, unicode):
-                message = message.encode("utf-8")
-            strmessages.append(message)
-        messages = ", ".join(strmessages)
-        Exception.__init__(self, messages)
+        assert isinstance(messages[0], unicode)  # Assumption: all of same type
+        joined = u", ".join(messages)
+        Exception.__init__(self, joined)
+        # Python 2.3 doesn't have .args
+        if not hasattr(self, "args"):
+            self.args = joined
 
 class SeriousFilterFailure(FilterFailure):
     """This exception signals that a Filter didn't pass, and the bad translation 
@@ -139,48 +143,49 @@ class CheckerConfig(object):
                     notranslatewords=None, musttranslatewords=None, validchars=None, 
                     punctuation=None, endpunctuation=None, ignoretags=None, 
                     canchangetags=None, criticaltests=None, credit_sources=None):
-        # we have to initialise empty lists properly (default arguments get reused)
-        if accelmarkers is None:
-            accelmarkers = []
-        if varmatches is None:
-            varmatches = []
-        if musttranslatewords is None:
-            musttranslatewords = []
-        if notranslatewords is None:
-            notranslatewords = []
+        # Init lists
+        self.accelmarkers = self._init_list(accelmarkers)
+        self.varmatches = self._init_list(varmatches)
+        self.criticaltests = self._init_list(criticaltests)
+        self.credit_sources = self._init_list(credit_sources)
+        # Lang data
         self.targetlanguage = targetlanguage
         self.updatetargetlanguage(targetlanguage)
         self.sourcelang = factory.getlanguage('en')
-        self.accelmarkers = accelmarkers
-        self.varmatches = varmatches
+        # Inits with default values
+        self.punctuation = self._init_default(data.forceunicode(punctuation),  self.lang.punctuation)
+        self.endpunctuation = self._init_default(data.forceunicode(endpunctuation), self.lang.sentenceend)
+        self.ignoretags = self._init_default(ignoretags, common_ignoretags)
+        self.canchangetags = self._init_default(canchangetags, common_canchangetags)
+        # Other data
         # TODO: allow user configuration of untranslatable words
-        self.notranslatewords = dict.fromkeys([data.forceunicode(key) for key in notranslatewords])
-        self.musttranslatewords = dict.fromkeys([data.forceunicode(key) for key in musttranslatewords])
+        self.notranslatewords = dict.fromkeys([data.forceunicode(key) for key in self._init_list(notranslatewords)])
+        self.musttranslatewords = dict.fromkeys([data.forceunicode(key) for key in self._init_list(musttranslatewords)])
         validchars = data.forceunicode(validchars)
         self.validcharsmap = {}
         self.updatevalidchars(validchars)
-        punctuation = data.forceunicode(punctuation)
-        if punctuation is None:
-            punctuation = self.lang.punctuation
-        self.punctuation = punctuation
-        endpunctuation = data.forceunicode(endpunctuation)
-        if endpunctuation is None:
-            endpunctuation = self.lang.sentenceend
-        self.endpunctuation = endpunctuation
-        if ignoretags is None:
-            self.ignoretags = common_ignoretags
-        else:
-            self.ignoretags = ignoretags
-        if canchangetags is None:
-            self.canchangetags = common_canchangetags
-        else:
-            self.canchangetags = canchangetags
-        if criticaltests is None:
-            criticaltests = []
-        self.criticaltests = criticaltests
-        if credit_sources is None:
-            credit_sources = []
-        self.credit_sources = credit_sources
+
+    def _init_list(self, list):
+        """initialise configuration paramaters that are lists
+
+        @type list: List
+        @param list: None (we'll initialise a blank list) or a list paramater
+        @rtype: List
+        """
+        if list is None:
+            list = []
+        return list
+
+    def _init_default(self, param, default):
+        """initialise parameters that can have default options
+
+        @param param: the user supplied paramater value
+        @param default: default values when param is not specified
+        @return: the paramater as specified by the user of the default settings
+        """
+        if param is None:
+            return default
+        return param
 
     def update(self, otherconfig):
         """combines the info in otherconfig into this config object"""
@@ -210,6 +215,18 @@ class CheckerConfig(object):
         """Updates the target language in the config to the given target language"""
         self.lang = factory.getlanguage(langcode)
 
+def cache_results(f):
+    def cached_f(self, param1):
+        key = (f.__name__, param1)
+        res_cache = self.results_cache
+        if key in res_cache:
+            return res_cache[key]
+        else:
+            value = f(self, param1)
+            res_cache[key] = value
+            return value
+    return cached_f
+
 class UnitChecker(object):
     """Parent Checker class which does the checking based on functions available 
     in derived classes."""
@@ -228,6 +245,8 @@ class UnitChecker(object):
             if callable(function):
                 self.helperfunctions[functionname] = function
         self.defaultfilters = self.getfilters(excludefilters, limitfilters)
+        
+        self.results_cache = {}
 
     def getfilters(self, excludefilters=None, limitfilters=None):
         """returns dictionary of available filters, including/excluding those in 
@@ -263,22 +282,31 @@ class UnitChecker(object):
     def filtervariables(self, str1):
         """filter out variables from str1"""
         return helpers.multifilter(str1, self.varfilters)
+    filtervariables = cache_results(filtervariables)
 
     def removevariables(self, str1):
         """remove variables from str1"""
         return helpers.multifilter(str1, self.removevarfilter)
+    removevariables = cache_results(removevariables)
 
     def filteraccelerators(self, str1):
         """filter out accelerators from str1"""
-        return helpers.multifilter(str1, self.accfilters)
+        return helpers.multifilter(str1, self.accfilters, None)
+    filteraccelerators = cache_results(filteraccelerators)
+
+    def filteraccelerators_by_list(self, str1, acceptlist=None):
+        """filter out accelerators from str1"""
+        return helpers.multifilter(str1, self.accfilters, acceptlist)
 
     def filterwordswithpunctuation(self, str1):
         """replaces words with punctuation with their unpunctuated equivalents"""
         return prefilters.filterwordswithpunctuation(str1)
+    filterwordswithpunctuation = cache_results(filterwordswithpunctuation)
 
     def filterxml(self, str1):
         """filter out XML from the string so only text remains"""
         return tag_re.sub("", str1)
+    filterxml = cache_results(filterxml)
 
     def run_test(self, test, unit):
         """Runs the given test on the given unit.
@@ -288,6 +316,7 @@ class UnitChecker(object):
 
     def run_filters(self, unit):
         """run all the tests in this suite, return failures as testname, message_or_exception"""
+        self.results_cache = {}
         failures = {}
         ignores = self.config.lang.ignoretests[:]
         functionnames = self.defaultfilters.keys()
@@ -305,7 +334,7 @@ class UnitChecker(object):
                 filterresult = self.run_test(filterfunction, unit)
             except FilterFailure, e:
                 filterresult = False
-                filtermessage = str(e).decode('utf-8')
+                filtermessage = e.args[0]
             except Exception, e:
                 if self.errorhandler is None:
                     raise ValueError("error in filter %s: %r, %r, %s" % \
@@ -319,6 +348,7 @@ class UnitChecker(object):
                 if functionname in self.preconditions:
                     for ignoredfunctionname in self.preconditions[functionname]:
                         ignores.append(ignoredfunctionname)
+        self.results_cache = {}
         return failures
 
 class TranslationChecker(UnitChecker):
@@ -334,11 +364,19 @@ class TranslationChecker(UnitChecker):
         
         Note that this can raise a FilterFailure as part of normal operation."""
         if self.hasplural:
+            filtermessages = []
+            filterresult = True
             for pluralform in unit.target.strings:
-                if not test(self.str1, pluralform):
-                    return False
+                try:
+                    if not test(self.str1, unicode(pluralform)):
+                        filterresult = False
+                except FilterFailure, e:
+                    filterresult = False
+                    filtermessages.append( unicode(e.args) )
+            if not filterresult and filtermessages:
+                raise FilterFailure(filtermessages)
             else:
-                return True
+                return filterresult
         else:
             return test(self.str1, self.str2)
 
@@ -409,20 +447,21 @@ class StandardChecker(TranslationChecker):
 
     def unchanged(self, str1, str2):
         """checks whether a translation is basically identical to the original string"""
-        str1 = self.filteraccelerators(str1)
-        str2 = self.filteraccelerators(str2)
-        if len(str1.strip()) == 0:
+        str1 = self.removevariables(self.filteraccelerators(str1)).strip()
+        str2 = self.removevariables(self.filteraccelerators(str2)).strip()
+        if len(str1) < 2:
             return True
         if str1.isupper() and str1 == str2:
             return True
         if self.config.notranslatewords:
             words1 = str1.split()
             if len(words1) == 1 and [word for word in words1 if word in self.config.notranslatewords]:
+            #currently equivalent to:
+            #   if len(words1) == 1 and words1[0] in self.config.notranslatewords:
+            #why do we only test for one notranslate word?
                 return True
-        str1 = self.removevariables(str1)
-        str2 = self.removevariables(str2)
-        if not (str1.strip().isdigit() or len(str1) < 2 or decoration.ispurepunctuation(str1.strip())) and (str1.strip().lower() == str2.strip().lower()):
-            raise FilterFailure("please translate")
+        if str1.isalpha() and str1.lower() == str2.lower():
+            raise FilterFailure(u"please translate")
         return True
 
     def blank(self, str1, str2):
@@ -455,17 +494,18 @@ class StandardChecker(TranslationChecker):
     def newlines(self, str1, str2):
         """checks whether newlines are consistent between the two strings"""
         if not helpers.countsmatch(str1, str2, ("\n", "\r")):
-            raise FilterFailure("line endings in original don't match line endings in translation")
+            raise FilterFailure(u"line endings in original don't match line endings in translation")
         else:
             return True
 
     def tabs(self, str1, str2):
         """checks whether tabs are consistent between the two strings"""
         if not helpers.countmatch(str1, str2, "\t"):
-            raise SeriousFilterFailure("tabs in original don't match tabs in translation")
+            raise SeriousFilterFailure(u"tabs in original don't match tabs in translation")
         else:
             return True
 
+
     def singlequoting(self, str1, str2):
         """checks whether singlequoting is consistent between the two strings"""
         str1 = self.filterwordswithpunctuation(self.filteraccelerators(self.filtervariables(str1)))
@@ -510,18 +550,38 @@ class StandardChecker(TranslationChecker):
 
     def printf(self, str1, str2):
         """checks whether printf format strings match"""
-        count1 = count2 = None
+        count1 = count2 = plural = None
+        # self.hasplural only set by run_filters, not always available
+        if 'hasplural' in self.__dict__:
+            plural = self.hasplural
         for var_num2, match2 in enumerate(printf_pat.finditer(str2)):
             count2 = var_num2 + 1
+            str2key = match2.group('key')
             if match2.group('ord'):
                 for var_num1, match1 in enumerate(printf_pat.finditer(str1)):
                     count1 = var_num1 + 1
                     if int(match2.group('ord')) == var_num1 + 1:
                         if match2.group('fullvar') != match1.group('fullvar'):
                             return 0
+            elif str2key:
+                str1key = None
+                for var_num1, match1 in enumerate(printf_pat.finditer(str1)):
+                    count1 = var_num1 + 1
+                    if match1.group('key') and str2key == match1.group('key'):
+                        str1key = match1.group('key')
+                        # '%.0s' "placeholder" in plural will match anything
+                        if plural and match2.group('fullvar') == '.0s':
+                            continue
+                        if match1.group('fullvar') != match2.group('fullvar'):
+                            return 0
+                if str1key == None:
+                    return 0
             else:
                 for var_num1, match1 in enumerate(printf_pat.finditer(str1)):
                     count1 = var_num1 + 1
+                    # '%.0s' "placeholder" in plural will match anything
+                    if plural and match2.group('fullvar') == '.0s':
+                        continue
                     if (var_num1 == var_num2) and (match1.group('fullvar') != match2.group('fullvar')):
                         return 0
 
@@ -539,24 +599,25 @@ class StandardChecker(TranslationChecker):
         str2 = self.filtervariables(str2)
         messages = []
         for accelmarker in self.config.accelmarkers:
-            counter = decoration.countaccelerators(accelmarker)
-            count1, countbad1 = counter(str1)
-            count2, countbad2 = counter(str2)
-            getaccel = decoration.getaccelerators(accelmarker)
+            counter1 = decoration.countaccelerators(accelmarker, self.config.sourcelang.validaccel)
+            counter2 = decoration.countaccelerators(accelmarker, self.config.lang.validaccel)
+            count1, countbad1 = counter1(str1)
+            count2, countbad2 = counter2(str2)
+            getaccel = decoration.getaccelerators(accelmarker, self.config.lang.validaccel)
             accel2, bad2 = getaccel(str2)
             if count1 == count2:
                 continue
             if count1 == 1 and count2 == 0:
                 if countbad2 == 1:
-                    messages.append("accelerator %s appears before an invalid accelerator character '%s' (eg. space)" % (accelmarker, bad2[0]))
+                    messages.append(u"accelerator %s appears before an invalid accelerator character '%s' (eg. space)" % (accelmarker, bad2[0]))
                 else:
-                    messages.append("accelerator %s is missing from translation" % accelmarker)
+                    messages.append(u"accelerator %s is missing from translation" % accelmarker)
             elif count1 == 0:
-                messages.append("accelerator %s does not occur in original and should not be in translation" % accelmarker)
+                messages.append(u"accelerator %s does not occur in original and should not be in translation" % accelmarker)
             elif count1 == 1 and count2 > count1:
-                messages.append("accelerator %s is repeated in translation" % accelmarker)
+                messages.append(u"accelerator %s is repeated in translation" % accelmarker)
             else:
-                messages.append("accelerator %s occurs %d time(s) in original and %d time(s) in translation" % (accelmarker, count1, count2))
+                messages.append(u"accelerator %s occurs %d time(s) in original and %d time(s) in translation" % (accelmarker, count1, count2))
         if messages:
             if "accelerators" in self.config.criticaltests:
                 raise SeriousFilterFailure(messages)
@@ -571,9 +632,9 @@ class StandardChecker(TranslationChecker):
 #            for variablestart, variableend in self.config.varmatches:
 #                error = accelerator + variablestart
 #                if str1.find(error) >= 0:
-#                    messages.append("original has an accelerated variable")
+#                    messages.append(u"original has an accelerated variable")
 #                if str2.find(error) >= 0:
-#                    messages.append("translation has an accelerated variable")
+#                    messages.append(u"translation has an accelerated variable")
 #        if messages:
 #            raise FilterFailure(messages)
 #        return True
@@ -608,9 +669,9 @@ class StandardChecker(TranslationChecker):
                 mismatch1.extend(vars1)
                 mismatch2.extend(vars2)
         if mismatch1:
-            messages.append("do not translate: %s" % ", ".join(mismatch1))
+            messages.append(u"do not translate: %s" % ", ".join(mismatch1))
         elif mismatch2:
-            messages.append("translation contains variables not in original: %s" % ", ".join(mismatch2))
+            messages.append(u"translation contains variables not in original: %s" % ", ".join(mismatch2))
         if messages and mismatch1:
             raise SeriousFilterFailure(messages)
         elif messages:
@@ -682,9 +743,9 @@ class StandardChecker(TranslationChecker):
             elif count2 > count1:
                 extra.append("'%s'" % bracket)
         if missing:
-            messages.append("translation is missing %s" % ", ".join(missing))
+            messages.append(u"translation is missing %s" % ", ".join(missing))
         if extra:
-            messages.append("translation has extra %s" % ", ".join(extra))
+            messages.append(u"translation has extra %s" % ", ".join(extra))
         if messages:
             raise FilterFailure(messages)
         return True
@@ -694,7 +755,7 @@ class StandardChecker(TranslationChecker):
         sentences1 = len(self.config.sourcelang.sentences(str1))
         sentences2 = len(self.config.lang.sentences(str2))
         if not sentences1 == sentences2:
-            raise FilterFailure("The number of sentences differ: %d versus %d" % (sentences1, sentences2))
+            raise FilterFailure(u"The number of sentences differ: %d versus %d" % (sentences1, sentences2))
         return True
 
     def options(self, str1, str2):
@@ -704,9 +765,9 @@ class StandardChecker(TranslationChecker):
             if word1 != "--" and word1.startswith("--") and word1[-1].isalnum():
                 parts = word1.split("=")
                 if not parts[0] in str2:
-                    raise FilterFailure("The option %s does not occur or is translated in the translation." % parts[0]) 
+                    raise FilterFailure(u"The option %s does not occur or is translated in the translation." % parts[0]) 
                 if len(parts) > 1 and parts[1] in str2:
-                    raise FilterFailure("The parameter %(param)s in option %(option)s is not translated." % {"param": parts[0], "option": parts[1]})
+                    raise FilterFailure(u"The parameter %(param)s in option %(option)s is not translated." % {"param": parts[1], "option": parts[0]})
         return True
 
     def startcaps(self, str1, str2):
@@ -728,10 +789,10 @@ class StandardChecker(TranslationChecker):
         # TODO: review this. The 'I' is specific to English, so it probably serves
         # no purpose to get sourcelang.sentenceend
         str1 = re.sub(u"[^%s]( I )" % self.config.sourcelang.sentenceend, " i ", str1)
-        capitals1 = helpers.filtercount(str1, type(str1).isupper)
-        capitals2 = helpers.filtercount(str2, type(str2).isupper)
-        alpha1 = helpers.filtercount(str1, type(str1).isalpha)
-        alpha2 = helpers.filtercount(str2, type(str2).isalpha)
+        capitals1 = helpers.filtercount(str1, unicode.isupper)
+        capitals2 = helpers.filtercount(str2, unicode.isupper)
+        alpha1 = helpers.filtercount(str1, unicode.isalpha)
+        alpha2 = helpers.filtercount(str2, unicode.isalpha)
         # Capture the all caps case
         if capitals1 == alpha1:
             return capitals2 == alpha2
@@ -739,7 +800,7 @@ class StandardChecker(TranslationChecker):
         if capitals1 == 0 or capitals1 == 1:
             return capitals2 == capitals1
         elif capitals1 < len(str1) / 10:
-            return capitals2 < len(str2) / 8
+            return capitals2 <= len(str2) / 8
         elif len(str1) < 10:
             return abs(capitals1 - capitals2) < 3
         elif capitals1 > len(str1) * 6 / 10:
@@ -757,6 +818,10 @@ class StandardChecker(TranslationChecker):
         str1 = self.filteraccelerators(self.filtervariables(str1))
         iter = self.config.lang.word_iter(str1)
         str2 = self.filteraccelerators(self.filtervariables(str2))
+        #TODO: strip XML? - should provide better error messsages
+        # see mail/chrome/messanger/smime.properties.po
+        #TODO: consider limiting the word length for recognising acronyms to 
+        #something like 5/6 characters
         for word in iter:
             if word.isupper() and len(word) > 1 and word not in allowed:
                 if str2.find(word) == -1:
@@ -785,14 +850,8 @@ class StandardChecker(TranslationChecker):
         #The above is full of strange quotes and things in utf-8 encoding.
         #single apostrophe perhaps problematic in words like "doesn't"
         for seperator in self.config.punctuation:
-            if isinstance(str1, unicode):
-                str1 = str1.replace(seperator, u" ")
-            else:
-                str1 = str1.replace(seperator.encode("utf-8"), " ")
-            if isinstance(str2, unicode):
-                str2 = str2.replace(seperator, u" ")
-            else:
-                str2 = str2.replace(seperator.encode("utf-8"), " ")
+            str1 = str1.replace(seperator, u" ")
+            str2 = str2.replace(seperator, u" ")
         words1 = self.filteraccelerators(str1).split()
         words2 = self.filteraccelerators(str2).split()
         stopwords = [word for word in words1 if word in self.config.notranslatewords and word not in words2]
@@ -825,9 +884,9 @@ class StandardChecker(TranslationChecker):
             return True
         invalid1 = str1.translate(self.config.validcharsmap)
         invalid2 = str2.translate(self.config.validcharsmap)
-        invalidchars = ["'%s' (\\u%04x)" % (invalidchar.encode('utf-8'), ord(invalidchar)) for invalidchar in invalid2 if invalidchar not in invalid1]
+        invalidchars = [u"'%s' (\\u%04x)" % (invalidchar, ord(invalidchar)) for invalidchar in invalid2 if invalidchar not in invalid1]
         if invalidchars:
-            raise FilterFailure("invalid chars: %s" % (", ".join(invalidchars)))
+            raise FilterFailure(u"invalid chars: %s" % (u", ".join(invalidchars)))
         return True
 
     def filepaths(self, str1, str2):
@@ -893,22 +952,23 @@ class StandardChecker(TranslationChecker):
         """checks words that don't pass a spell check"""
         if not self.config.targetlanguage:
             return True
-        str1 = self.filteraccelerators(self.filtervariables(str1))
-        str2 = self.filteraccelerators(self.filtervariables(str2))
+        if not spelling.available:
+            return True
+        str1 = self.filteraccelerators_by_list(self.filtervariables(str1), self.config.sourcelang.validaccel)
+        str2 = self.filteraccelerators_by_list(self.filtervariables(str2), self.config.lang.validaccel)
         ignore1 = []
         messages = []
         for word, index, suggestions in spelling.check(str1, lang="en"):
             ignore1.append(word)
         for word, index, suggestions in spelling.check(str2, lang=self.config.targetlanguage):
+            if word in self.config.notranslatewords:
+                continue
             if word in ignore1:
                 continue
             # hack to ignore hyphenisation rules
             if word in suggestions:
                 continue
-            if isinstance(str2, unicode) or isinstance(str1, unicode):
-                messages.append(u"check spelling of %s (could be %s)" % (word, u" / ".join(suggestions)))
-            else:
-                messages.append("check spelling of %s (could be %s)" % (word, " / ".join(suggestions)))
+            messages.append(u"check spelling of %s (could be %s)" % (word, u" / ".join(suggestions)))
         if messages:
             raise FilterFailure(messages)
         return True
@@ -1086,8 +1146,8 @@ def runtests(str1, str2, ignorelist=()):
     unit.target = str2
     checker = StandardChecker(excludefilters=ignorelist)
     failures = checker.run_filters(unit)
-    for testname, message in failures:
-        print "failure: %s: %s\n  %r\n  %r" % (testname, message, str1, str2)
+    for test in failures:
+        print "failure: %s: %s\n  %r\n  %r" % (test, failures[test], str1, str2)
     return failures
 
 def batchruntests(pairs):
