@@ -3,36 +3,46 @@ from translate.filters import checks
 from translate.lang import data
 from translate.storage import po 
 
+def strprep(str1, str2, message=None):
+    return data.forceunicode(str1), data.forceunicode(str2), data.forceunicode(message)
+
 def passes(filterfunction, str1, str2):
     """returns whether the given strings pass on the given test, handling FilterFailures"""
-    str1 = data.forceunicode(str1)
-    str2 = data.forceunicode(str2)
+    str1, str2, no_message = strprep(str1, str2)
     try:
         filterresult = filterfunction(str1, str2)
     except checks.FilterFailure, e:
         filterresult = False
     return filterresult
 
-def fails(filterfunction, str1, str2):
+def fails(filterfunction, str1, str2, message=None):
     """returns whether the given strings fail on the given test, handling only FilterFailures"""
-    str1 = data.forceunicode(str1)
-    str2 = data.forceunicode(str2)
+    str1, str2, message = strprep(str1, str2, message)
     try:
         filterresult = filterfunction(str1, str2)
     except checks.SeriousFilterFailure, e:
         filterresult = True
     except checks.FilterFailure, e:
-        filterresult = False
+        if message:
+            exc_message = e.args[0]
+            filterresult = exc_message != message
+            print exc_message.encode('utf-8')
+        else:
+            filterresult = False
     return not filterresult
 
-def fails_serious(filterfunction, str1, str2):
+def fails_serious(filterfunction, str1, str2, message=None):
     """returns whether the given strings fail on the given test, handling only SeriousFilterFailures"""
-    str1 = data.forceunicode(str1)
-    str2 = data.forceunicode(str2)
+    str1, str2, message = strprep(str1, str2, message)
     try:
         filterresult = filterfunction(str1, str2)
     except checks.SeriousFilterFailure, e:
-        filterresult = False
+        if message:
+            exc_message = e.args[0]
+            filterresult = exc_message != message
+            print exc_message.encode('utf-8')
+        else:
+            filterresult = False
     return not filterresult
 
 
@@ -64,6 +74,13 @@ def test_accelerator_markers():
     assert gnomechecker.config.accelmarkers == ["_"]
     kdechecker = checks.KdeChecker()
     assert kdechecker.config.accelmarkers == ["&"]
+
+def test_messages():
+    """test that our helpers can check for messages and that these error messages can contain Unicode"""
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(validchars='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'))
+    assert fails(stdchecker.validchars, "Some unexpected characters", "©", "invalid chars: '©' (\\u00a9)")
+    stdchecker = checks.StandardChecker()
+    assert fails_serious(stdchecker.escapes, r"A tab", r"'n Ṱab\t", r"""escapes in original () don't match escapes in translation ('Ṱab\t')""")
     
 def test_accelerators():
     """tests accelerators"""
@@ -83,6 +100,7 @@ def test_accelerators():
     assert passes(gnomechecker.accelerators, "_File", "_Fayile") 
     assert fails(gnomechecker.accelerators, "_File", "Fayile") 
     assert fails(gnomechecker.accelerators, "File", "_Fayile") 
+    assert fails(gnomechecker.accelerators, "_File", "_Fayil_e") 
     mozillachecker = checks.MozillaChecker()
     assert passes(mozillachecker.accelerators, "&File", "&Fayile") 
     assert fails_serious(mozillachecker.accelerators, "&File", "Fayile") 
@@ -97,6 +115,9 @@ def test_accelerators():
 
     # We don't want an accelerator for letters with a diacritic
     assert fails(ooochecker.accelerators, "F~ile", "L~êer")
+    # Bug 289: accept accented accelerator characters
+    afchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&", targetlanguage="fi"))
+    assert passes(afchecker.accelerators, "&Reload Frame", "P&äivitä kehys")
     # Problems:
     # Accelerator before variable - see test_acceleratedvariables
 
@@ -166,6 +187,9 @@ def test_doublequoting():
     assert passes(frchecker.doublequoting, "Do \"this\"", "Do « this »")
     assert fails(frchecker.doublequoting, "Do \"this\"", "Do « this » « this »")
     
+    vichecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="vi"))
+    assert passes(vichecker.doublequoting, 'Save "File"', u"Lưu « Tập tin »")
+    
 def test_doublespacing():
     """tests double spacing"""
     stdchecker = checks.StandardChecker()
@@ -207,6 +231,9 @@ def test_endpunc():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='zh'))
     assert passes(stdchecker.endpunc, "To activate your account, follow this link:\n", u"要啟用戶口，請瀏覽這個鏈結：\n")
 
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='vi'))
+    assert passes(stdchecker.endpunc, "Do you want to delete the XX dialog?", u"Bạn có muốn xoá hộp thoại XX không?")
+
 def test_endwhitespace():
     """tests endwhitespace"""
     stdchecker = checks.StandardChecker()
@@ -397,6 +424,14 @@ def test_printf():
     assert passes(stdchecker.printf, "String %1$s and number %2$d", "String %1$s en nommer %2$d")
     assert passes(stdchecker.printf, "String %s and number %d", "Nommer %2$d and string %1$s")
     assert fails(stdchecker.printf, "String %s and number %d", "Nommer %1$d and string %2$s")
+    # checking python format strings
+    assert passes(stdchecker.printf, "String %(1)s and number %(2)d", "Nommer %(2)d en string %(1)s")
+    assert passes(stdchecker.printf, "String %(str)s and number %(num)d", "Nommer %(num)d en string %(str)s")
+    assert fails(stdchecker.printf, "String %(str)s and number %(num)d", "Nommer %(nommer)d en string %(str)s")
+    assert fails(stdchecker.printf, "String %(str)s and number %(num)d", "Nommer %(num)d en string %s")
+    # checking omitted plural format string placeholder %.0s
+    stdchecker.hasplural = 1
+    assert passes(stdchecker.printf, "%d plurals", "%.0s plural")
 
 def test_puncspacing():
     """tests spacing after punctuation"""
@@ -419,6 +454,7 @@ def test_purepunc():
     assert fails(stdchecker.purepunc, ".", " ")
     assert fails(stdchecker.purepunc, "Find", "'")
     assert fails(stdchecker.purepunc, "'", "Find")
+    assert passes(stdchecker.purepunc, "year measurement template|2000", "2000")
 
 def test_sentencecount():
     """tests sentencecount messages"""
@@ -453,6 +489,7 @@ def test_singlequoting():
     # Real examples
     assert passes(stdchecker.singlequoting, "A nickname that identifies this publishing site (e.g.: 'MySite')", "Vito ro duvulela leri tirhisiwaka ku kuma sayiti leri ro kandziyisa (xik.: 'Sayiti ra Mina')")
     assert passes(stdchecker.singlequoting, "isn't", "ayikho")
+    assert passes(stdchecker.singlequoting, "Required (can't send message unless all recipients have certificates)", "Verlang (kan nie boodskappe versend tensy al die ontvangers sertifikate het nie)")
     # Afrikaans 'n
     assert passes(stdchecker.singlequoting, "Please enter a different site name.", "Tik 'n ander werfnaam in.")
     assert passes(stdchecker.singlequoting, "\"%name%\" already exists. Please enter a different site name.", "\"%name%\" bestaan reeds. Tik 'n ander werfnaam in.")
@@ -462,6 +499,10 @@ def test_singlequoting():
     ooochecker = checks.OpenOfficeChecker()
     assert passes(ooochecker.singlequoting, "~Don't import anything", "~Moenie enigiets invoer nie")
 
+    vichecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="vi"))
+    assert passes(vichecker.doublequoting, "Save 'File'", u"Lưu « Tập tin »")
+    assert passes(vichecker.doublequoting, "Save `File'", u"Lưu « Tập tin »")
+
 def test_simplecaps():
     """tests simple caps"""
     # Simple caps is a very vauge test so the checks here are mostly for obviously fixable problem
@@ -492,6 +533,16 @@ def test_spellcheck():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="af"))
     assert passes(stdchecker.spellcheck, "Great trek", "Groot trek")
     assert fails(stdchecker.spellcheck, "Final deadline", "End of the road")
+    # Bug 289: filters accelerators before spell checking
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&", targetlanguage="fi"))
+    assert passes(stdchecker.spellcheck, "&Reload Frame", "P&äivitä kehys")
+    # Ensure we don't check notranslatewords
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="af"))
+    assert fails(stdchecker.spellcheck, "Mozilla is wonderful", "Mozillaaa is wonderlik")
+    # We should pass the test if the "error" occurs in the English
+    assert passes(stdchecker.spellcheck, "Mozilla is wonderful", "Mozilla is wonderlik")
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="af", notranslatewords=["Mozilla"]))
+    assert passes(stdchecker.spellcheck, "Mozilla is wonderful", "Mozilla is wonderlik")
 
 def test_startcaps():
     """tests starting capitals"""
@@ -633,6 +684,8 @@ def test_variables_mozilla():
     assert fails_serious(mozillachecker.variables, "About $ProgramName$", "Oor $NaamVanProgam$")
     assert passes(mozillachecker.variables, "About $_CLICK", "Oor $_CLICK")
     assert fails_serious(mozillachecker.variables, "About $_CLICK", "Oor $_KLIK")
+    assert passes(mozillachecker.variables, "About $_CLICK and more", "Oor $_CLICK en meer")
+    assert fails_serious(mozillachecker.variables, "About $_CLICK and more", "Oor $_KLIK en meer")
     assert passes(mozillachecker.variables, "About $(^NameDA)", "Oor $(^NameDA)")
     assert fails_serious(mozillachecker.variables, "About $(^NameDA)", "Oor $(^NaamDA)")
     # Double variable problem
