@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # 
-# Copyright 2007 Zuza Software Foundation
+# Copyright 2007-2008 Zuza Software Foundation
 # 
 # This file is part of translate.
 #
@@ -21,39 +21,40 @@
 
 """This module contains all the common features for languages.
 
-Supported features:
-language code (km, af)
-language name (Khmer, Afrikaans)
-Plurals
-  Number of plurals (nplurals)
-  Plural equation
-pofilter tests to ignore
-
-Segmentation
-  characters
-  words
-  sentences
-
-TODO:
-Ideas for possible features:
-
-Language-Team information
-
-Segmentation
-  phrases
-
-Punctuation
-  End of sentence
-  Start of sentence
-  Middle of sentence
-  Quotes
-    single
-    double
-
-Valid characters
-Accelerator characters
-Special characters
-Direction (rtl or ltr)
+   Supported features
+   ==================
+     - language code (km, af)
+     - language name (Khmer, Afrikaans)
+     - Plurals
+       - Number of plurals (nplurals)
+       - Plural equation
+     - pofilter tests to ignore
+   
+   Segmentation
+   ------------
+     - characters
+     - words
+     - sentences
+   
+   TODOs and Ideas for possible features
+   =====================================
+     - Language-Team information
+     - Segmentation
+       - phrases
+   
+   Punctuation
+   -----------
+     - End of sentence
+     - Start of sentence
+     - Middle of sentence
+     - Quotes
+       - single
+       - double
+   
+     - Valid characters
+     - Accelerator characters
+     - Special characters
+     - Direction (rtl or ltr)
 """
 
 from translate.lang import data
@@ -66,7 +67,7 @@ class Common(object):
     """The ISO 639 language code, possibly with a country specifier or other 
     modifier.
     
-    Examples:
+    Examples::
         km
         pt_BR
         sr_YU@Latn
@@ -75,10 +76,10 @@ class Common(object):
     fullname = ""
     """The full (English) name of this language.
 
-    Dialect codes should have the form of 
-      Khmer
-      Portugese (Brazil)
-      #TODO: sr_YU@Latn?
+       Dialect codes should have the form of 
+         - Khmer
+         - Portugese (Brazil)
+         - TODO: sr_YU@Latn?
     """
     
     nplurals = 0
@@ -96,6 +97,8 @@ class Common(object):
     See U{http://www.gnu.org/software/gettext/manual/html_node/gettext_150.html}.
     Also see data.py
     """
+    # Don't change these defaults of nplurals or pluralequation willy-nilly:
+    # some code probably depends on these for unrecognised languages
     
     listseperator = u", "
     """This string is used to seperate lists of textual elements. Most 
@@ -152,7 +155,8 @@ class Common(object):
                             """ % sentenceend, re.VERBOSE)
     
     puncdict = {}
-    """A dictionary of punctuation transformation rules that can be used by punctranslate()."""
+    """A dictionary of punctuation transformation rules that can be used by 
+    punctranslate()."""
 
     ignoretests = []
     """List of pofilter tests for this language that must be ignored."""
@@ -162,20 +166,36 @@ class Common(object):
 
     This doesn't need to be supplied, but will be used if it exists."""
 
-    def __init__(self, code):
-        """This constructor is used if we need to instantiate an abject (not 
-        the usual setup). This will mostly when the factory is asked for a
-        language for which we don't have a dedicated class."""
-        self.code = code or ""
+    _languages = {}
+
+    validaccel = None
+    """Characters that can be used as accelerators (access keys) i.e. Alt+X 
+    where X is the accelerator.  These can include combining diacritics as 
+    long as they are accessible from the users keyboard in a single keystroke,
+    but normally they would be at least precomposed characters. All characters,
+    lower and upper, are included in the list."""
+
+    def __new__(cls, code):
+        """This returns the language class for the given code, following a 
+        singleton like approach (only one object per language)."""
+        code = code or ""
+        # First see if a language object for this code already exists
+        if code in cls._languages:
+            return cls._languages[code]
+        # No existing language. Let's build a new one and keep a copy
+        language = cls._languages[code] = object.__new__(cls)
+
+        language.code = code
         while code:
             langdata = data.languages.get(code, None)
             if langdata:
-                self.fullname, self.nplurals, self.pluralequation = langdata
+                language.fullname, language.nplurals, language.pluralequation = langdata
                 break
             code = data.simplercode(code)
         if not code:
 #            print >> sys.stderr, "Warning: No information found about language code %s" % code
             pass
+        return language
 
     def __repr__(self):
         """Give a simple string representation without address information to 
@@ -189,11 +209,15 @@ class Common(object):
         """Converts the punctuation in a string according to the rules of the 
         language."""
 #        TODO: look at po::escapeforpo() for performance idea
+        if not text:
+            return text
         for source, target in cls.puncdict.iteritems():
             text = text.replace(source, target)
         # Let's account for cases where a punctuation symbol plus a space is 
-        # replaced, but the space won't exist at the end of a message
-        if text and text[-1] + " " in cls.puncdict:
+        # replaced, but the space won't exist at the end of a message.
+        # As a simple improvement for messages ending in ellipses (...), we
+        # test that the last character is different from the second last
+        if (text[-1] + " " in cls.puncdict) and (text[-2] != text[-1]):
             text = text[:-1] + cls.puncdict[text[-1] + " "]
         return text
     punctranslate = classmethod(punctranslate)
@@ -232,6 +256,7 @@ class Common(object):
     def sentence_iter(cls, text, strip=True):
         """Returns an iterator over the sentences in text."""
         lastmatch = 0
+        text = text or ""
         iter = cls.sentencere.finditer(text)
         for item in iter:
             lastmatch = item.end()
