@@ -71,6 +71,7 @@ languages = {
 'km': ('Khmer', 1, '0'),
 'ko': ('Korean', 1, '0'),
 'ku': ('Kurdish', 2, '(n != 1)'),
+'ky': ('Kyrgyz', 1, '0'),
 'lb': ('Letzeburgesch', 2, '(n != 1)'),
 'ln': ('Lingala', 2, '(n > 1)'),
 'lt': ('Lithuanian', 3, '(n%10==1 && n%100!=11 ? 0 : n%10>=2 && (n%100<10 || n%100>=20) ? 1 : 2)'),
@@ -140,17 +141,27 @@ import re
 iso639 = {}
 iso3166 = {}
 
-dialectre = re.compile(r"([^(\s]+)\s*\(([^)]+)\)")
+langcode_re = re.compile("^[a-z]{2,3}([_-][A-Z]{2,3}|)(@[a-zA-Z0-9]+|)$")
+variant_re = re.compile("^[_-][A-Z]{2,3}(@[a-zA-Z0-9]+|)$")
+
+def languagematch(languagecode, otherlanguagecode):
+    """matches a languagecode to another, ignoring regions in the second"""
+    if languagecode is None:
+      return langcode_re.match(otherlanguagecode)
+    return languagecode == otherlanguagecode or \
+      (otherlanguagecode.startswith(languagecode) and variant_re.match(otherlanguagecode[len(languagecode):]))
+
+dialect_name_re = re.compile(r"([^(\s]+)\s*\(([^)]+)\)")
 
 def tr_lang(langcode):
-    """Gives a function that can translate a language name, even in the form 
-        "language (country)"
-    into the language with iso code langcode."""
+    """Gives a function that can translate a language name, even in the form::
+           "language (country)"
+       into the language with iso code langcode."""
     langfunc = gettext_lang(langcode)
     countryfunc = gettext_country(langcode)
 
     def handlelanguage(name):
-        match = dialectre.match(name)
+        match = dialect_name_re.match(name)
         if match:
             language, country = match.groups()
             return u"%s (%s)" % (langfunc(language), countryfunc(country))
@@ -178,11 +189,14 @@ def gettext_country(langcode):
 def normalize(string, normal_form="NFC"):
     """Return a unicode string in its normalized form
     
-    @param sting: The string to be normalized
-    @param normal_form: NFC (default), NFD, NFCK, NFDK
-    @return: Normalized string
+       @param string: The string to be normalized
+       @param normal_form: NFC (default), NFD, NFCK, NFDK
+       @return: Normalized string
     """
-    return unicodedata.normalize(normal_form, string)
+    if string is None:
+        return None
+    else:
+        return unicodedata.normalize(normal_form, string)
 
 def forceunicode(string):
     """Helper method to ensure that the parameter becomes unicode if not yet"""
