@@ -28,10 +28,6 @@ from translate.lang import common
 
 class fa(common.Common):
     """This class represents Persian."""
-    code = "fa"
-    fullname = "Persian"
-    nplurals = 1
-    pluralequation = "0"
 
     listseperator = u"، "
 
