@@ -27,12 +27,29 @@ For more information, see U{http://en.wikipedia.org/wiki/French_language}
 from translate.lang import common
 import re
 
+def guillemets(text):
+    def convertquotation(match):
+        prefix = match.group(1)
+        # Let's see that we didn't perhaps match an XML tag property like
+        # <a href="something">
+        if prefix == u"=":
+            return match.group(0)
+        return u"%s« %s »" % (prefix, match.group(2))
+
+    # Check that there is an even number of double quotes, otherwise it is
+    # probably not safe to convert them.
+    if text.count(u'"') % 2 == 0:
+        text = re.sub('(.|^)"([^"]+)"', convertquotation, text)
+    singlecount = text.count(u"'") 
+    if singlecount:
+        if singlecount % 2 == 0:
+            text = re.sub("(.|^)'([^']+)'", convertquotation, text)
+        if singlecount == text.count(u'`'):
+            text = re.sub("(.|^)`([^']+)'", convertquotation, text)
+    return text
+
 class fr(common.Common):
     """This class represents French."""
-    code = "fr"
-    fullname = "French"
-    nplurals = 2
-    pluralequation = "(n > 1)"
 
     # According to http://french.about.com/library/writing/bl-punctuation.htm, 
     # in French, a space is required both before and after all two- (or more) 
@@ -51,19 +68,8 @@ class fr(common.Common):
             - Double spaces might be introduced
         """
         text = super(cls, cls).punctranslate(text)
-
-        def convertquotation(match):
-            prefix = match.group(1)
-            # Let's see that we didn't perhaps match an XML tag property like
-            # <a href="something">
-            if prefix == u"=":
-                return match.group(0)
-            return u"%s« %s »" % (prefix, match.group(2))
-        
-        # Check that there is an even number of double quotes, otherwise it is
-        # probably not safe to convert them.
-        if text.count(u'"') % 2 == 0:
-            text = re.sub('(.|^)"([^"]+)"', convertquotation, text)
-
-        return text
+        # We might get problems where we got a space in URIs such as
+        # http ://
+        text = text.replace(u" ://", "://")
+        return guillemets(text)
     punctranslate = classmethod(punctranslate)
