@@ -9,6 +9,12 @@ def test_punctranslate():
     assert language.punctranslate(u"Document loaded") == u"Document loaded"
     assert language.punctranslate(u"Document loaded.") == u"Document loaded।"
 
+def test_country_code():
+    """Tests that we get the correct one even if a country code is attached to
+    a special code being a reserved word in Python (like 'or')."""
+    language = factory.getlanguage('or-IN')
+    assert language.fullname == "Oriya"
+
 def test_sentences():
     """Tests basic functionality of sentence segmentation."""
     language = factory.getlanguage('or')
