@@ -37,7 +37,7 @@ def python_distance(a, b, stopvalue=-1):
     if stopvalue == -1:
         stopvalue = l2
     current = range(l1+1)
-    for i in range(1,l2+1):
+    for i in range(1, l2+1):
         previous, current = current, [i]+[0]*l1
         least = l2
         for j in range(1, l1 + 1):
@@ -62,7 +62,7 @@ def native_distance(a, b, stopvalue=0):
     
     Note that this does not support arbitrary sequence types, but only 
     string types."""
-    return Levenshtein.distance(a,b)
+    return Levenshtein.distance(a, b)
 
 try:
     import Levenshtein as Levenshtein
@@ -97,32 +97,31 @@ class LevenshteinComparer:
 
     def similarity_real(self, a, b, stoppercentage=40):
         """Returns the similarity between a and b based on Levenshtein distance. It
-    can stop prematurely as soon as it sees that a and b will be no simmilar than
-    the percentage specified in stoppercentage.
+           can stop prematurely as soon as it sees that a and b will be no simmilar than
+           the percentage specified in stoppercentage.
 
-    The Levenshtein distance is calculated, but the following should be noted:
-        * Only the first MAX_LEN characters are considered. Long strings differing
-          at the end will therefore seem to match better than they should. See the
-          use of the variable penalty to lessen the effect of this.
-        * Strings with widely different lengths give the opportunity for shortcut.
-          This is by definition of the Levenshtein distance: the distance will be 
-          at least as much as the difference in string length.
-        * Calculation is stopped as soon as a similarity of stoppercentage becomes
-          unattainable. See the use of the variable stopvalue.
-        * Implementation uses memory O(min(len(a), len(b))
-        * Excecution time is O(len(a)*len(b))
+           The Levenshtein distance is calculated, but the following should be noted:
+               - Only the first MAX_LEN characters are considered. Long strings differing
+                 at the end will therefore seem to match better than they should. See the
+                 use of the variable penalty to lessen the effect of this.
+               - Strings with widely different lengths give the opportunity for shortcut.
+                 This is by definition of the Levenshtein distance: the distance will be 
+                 at least as much as the difference in string length.
+               - Calculation is stopped as soon as a similarity of stoppercentage becomes
+                 unattainable. See the use of the variable stopvalue.
+               - Implementation uses memory O(min(len(a), len(b))
+               - Excecution time is O(len(a)*len(b))
     """
         l1, l2 = len(a), len(b)
         if l1 == 0 or l2 == 0:
             return 0
         #Let's make l1 the smallest
         if l1 > l2:
-            l1,l2 = l2,l1
+            l1, l2 = l2, l1
             a, b = b, a
             
         #maxsimilarity is the maximum similarity that can be attained as constrained
         #by the difference in string length
-        assert 0 <= stoppercentage <= 100
         maxsimilarity = 100 - 100.0*abs(l1 - l2)/l2
         if maxsimilarity < stoppercentage:
             return maxsimilarity * 1.0
@@ -154,5 +153,5 @@ class LevenshteinComparer:
 if __name__ == "__main__":
     from sys import argv
     comparer = LevenshteinComparer()
-    print "Similarity:\n%s" % comparer.similarity(argv[1],argv[2], 50)
+    print "Similarity:\n%s" % comparer.similarity(argv[1], argv[2], 50)
 
