@@ -39,7 +39,7 @@ def sourcelencmp(x, y):
     ylen = sourcelen(y)
     return cmp(xlen, ylen)
 
-class matcher:
+class matcher(object):
     """A class that will do matching and store configuration for the matching process"""
     def __init__(self, store, max_candidates=10, min_similarity=75, max_length=70, comparer=None, usefuzzy=False):
         """max_candidates is the maximum number of candidates that should be assembled,
@@ -147,25 +147,30 @@ class matcher:
         translation memory. If self.addpercentage is true (default) the match 
         quality is given as a percentage in the notes.
         """
-        bestcandidates = [(0.0,None)]*self.MAX_CANDIDATES
-        heapq.heapify(bestcandidates)
+        bestcandidates = [(0.0, None)]*self.MAX_CANDIDATES
         #We use self.MIN_SIMILARITY, but if we already know we have max_candidates
         #that are better, we can adjust min_similarity upwards for speedup
         min_similarity = self.MIN_SIMILARITY
         
         # We want to limit our search in self.candidates, so we want to ignore
-        # all units with a source string that is too short or too long
+        # all units with a source string that is too short or too long. We use
+        # a binary search to find the shortest string, from where we start our
+        # search in the candidates.
 
         # minimum source string length to be considered
         startlength = self.getstartlength(min_similarity, text)
         startindex = 0
-        for index, candidate in enumerate(self.candidates.units):
-            if len(candidate.source) >= startlength:
-                startindex = index
-                break
+        endindex = len(self.candidates.units)
+        while startindex < endindex:
+            mid = (startindex + endindex) // 2
+            if sourcelen(self.candidates.units[mid]) < startlength:
+                startindex = mid + 1
+            else:
+                endindex = mid
         
         # maximum source string length to be considered
         stoplength = self.getstoplength(min_similarity, text) 
+        lowestscore = 0
 
         for candidate in self.candidates.units[startindex:]:
             cmpstring = candidate.source
@@ -174,12 +179,13 @@ class matcher:
             similarity = self.comparer.similarity(text, cmpstring, min_similarity)
             if similarity < min_similarity:
                 continue
-            lowestscore = bestcandidates[0][0]
             if similarity > lowestscore:
-                targetstring = candidate.target
                 heapq.heapreplace(bestcandidates, (similarity, candidate))
-                if min_similarity < bestcandidates[0][0]:
-                    min_similarity = bestcandidates[0][0]
+                lowestscore = bestcandidates[0][0]
+                if lowestscore >= 100:
+                    break
+                if min_similarity < lowestscore:
+                    min_similarity = lowestscore
                     stoplength = self.getstoplength(min_similarity, text) 
         
         #Remove the empty ones:
@@ -189,6 +195,7 @@ class matcher:
         bestcandidates = filter(notzero, bestcandidates)
         #Sort for use as a general list, and reverse so the best one is at index 0
         bestcandidates.sort()
+        # We reverse as separate step for compatibility with Python 2.3
         bestcandidates.reverse()
         return self.buildunits(bestcandidates)
 
