@@ -44,6 +44,7 @@ import os
 import pypo
 import re
 import sys
+import tempfile
 
 lsep = " "
 """Seperator for #: entries"""
@@ -157,6 +158,20 @@ def unquotefrompo(postr, joinwithlinebreak=False):
 def encodingToUse(encoding):
     return pypo.encodingToUse(encoding)
 
+def get_libgettextpo_version():
+    """Returns the libgettextpo version
+
+       @rtype: three-value tuple
+       @return: libgettextpo version in the following format::
+           (major version, minor version, subminor version)
+    """
+    libversion = c_long.in_dll(gpo, 'libgettextpo_version')
+    major = libversion.value >> 16
+    minor = libversion.value >> 8
+    subminor = libversion.value - (major << 16) - (minor << 8)
+    return major, minor, subminor
+
+
 class pounit(pocommon.pounit):
     def __init__(self, source=None, encoding='utf-8', gpo_message=None):
         self._encoding = encoding
@@ -237,16 +252,23 @@ class pounit(pocommon.pounit):
         return multi
 
     def settarget(self, target):
+        # for plural strings: convert 'target' into a list
         if self.hasplural():
             if isinstance(target, multistring):
                 target = target.strings
             elif isinstance(target, basestring):
                 target = [target]
+        # for non-plurals: check number of items in 'target'
         elif isinstance(target,(dict, list)):
             if len(target) == 1:
                 target = target[0]
             else:
                 raise ValueError("po msgid element has no plural but msgstr has %d elements (%s)" % (len(target), target))
+        # empty the previous list of messages
+        # TODO: the "pypo" implementation does not remove the previous items of
+        #   the target, if self.target == target (essentially: comparing only
+        #   the first item of a plural string with the single new string)
+        #   Maybe this behaviour should be unified.
         if isinstance(target, (dict, list)):
             i = 0
             message = gpo.po_message_msgstr_plural(self._gpo_message, i)
@@ -254,15 +276,18 @@ class pounit(pocommon.pounit):
                 gpo.po_message_set_msgstr_plural(self._gpo_message, i, None)
                 i += 1
                 message = gpo.po_message_msgstr_plural(self._gpo_message, i)
+        # add the items of a list
         if isinstance(target, list):
             for i in range(len(target)):
                 targetstring = target[i]
                 if isinstance(targetstring, unicode):
                     targetstring = targetstring.encode(self._encoding)
                 gpo.po_message_set_msgstr_plural(self._gpo_message, i, targetstring)
+        # add the values of a dict
         elif isinstance(target, dict):
             for i, targetstring in enumerate(target.itervalues()):
                 gpo.po_message_set_msgstr_plural(self._gpo_message, i, targetstring)
+        # add a single string
         else:
             if isinstance(target, unicode):
                 target = target.encode(self._encoding)
@@ -298,14 +323,15 @@ class pounit(pocommon.pounit):
             comments = gpo.po_message_extracted_comments(self._gpo_message)
         else:
             raise ValueError("Comment type not valid")
-        # FIXME this fixes a bug in Gettext that returns leading space with comments
-        if comments:
-            comments = "\n".join([line[1:] for line in comments.split("\n")])
+
+        if comments and get_libgettextpo_version() < (0, 17, 0):
+            comments = "\n".join([line.strip() for line in comments.split("\n")])
         # Let's drop the last newline
         return comments[:-1].decode(self._encoding)
 
     def addnote(self, text, origin=None, position="append"):
-        if not text:
+        # ignore empty strings and strings without non-space characters
+        if (not text) or (not text.strip()):
             return
         text = data.forceunicode(text)
         oldnotes = self.getnotes(origin)
@@ -326,11 +352,12 @@ class pounit(pocommon.pounit):
                 newnotes = text + '\n' + oldnotes
         else:
             newnotes = "\n".join([line.rstrip() for line in text.split("\n")])
-        # FIXME; workaround the need for leading spaces when adding comments to PO files in libgettexpo
+
         if newnotes:
             newlines = []
+            needs_space = get_libgettextpo_version() < (0, 17, 0)
             for line in newnotes.split("\n"):
-                if line:
+                if line and needs_space:
                     newlines.append(" " + line)
                 else:
                     newlines.append(line)
@@ -576,14 +603,11 @@ class pofile(pocommon.pofile):
         outputstring = ""
         if self._gpo_memory_file:
             obsolete_workaround()
-            outputfile = os.tmpnam()
-            f = open(outputfile, "w")
-            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, outputfile, xerror_handler)
-            f.close()
-            f = open(outputfile, "r")
+            f = tempfile.NamedTemporaryFile(prefix='translate', suffix='.po')
+            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, f.name, xerror_handler)
+            f.seek(0)
             outputstring = f.read()
             f.close()
-            os.remove(outputfile)
         return outputstring
 
     def isempty(self):
@@ -606,23 +630,27 @@ class pofile(pocommon.pofile):
             self.filename = input.name
         elif not getattr(self, 'filename', ''):
             self.filename = ''
+
         if hasattr(input, "read"):
             posrc = input.read()
             input.close()
             input = posrc
+
         needtmpfile = not os.path.isfile(input)
         if needtmpfile:
             # This is not a file - we write the string to a temporary file
-            tmpfile = os.tmpnam()
-            f = open(tmpfile, "w")
-            f.write(input)
-            f.close()
-            input = tmpfile
+            fd, fname = tempfile.mkstemp(prefix='translate', suffix='.po')
+            os.write(fd, input)
+            input = fname
+            os.close(fd)
+
         self._gpo_memory_file = gpo.po_file_read_v3(input, xerror_handler)
         if self._gpo_memory_file is None:
             print >> sys.stderr, "Error:"
+
         if needtmpfile:
-            os.remove(tmpfile)
+            os.remove(input)
+
         # Handle xerrors here
         self._header = gpo.po_file_domain_header(self._gpo_memory_file, None)
         if self._header:
