@@ -23,12 +23,7 @@
 or entire files (csvfile) for use with localisation
 """
 
-try:
-    # try to import the standard csv module, included from Python 2.3
-    import csv
-except:
-    # if it doesn't work, use our local copy of it...
-    from translate.misc import csv
+import csv
 
 from translate.misc import sparse
 from translate.storage import base
@@ -37,7 +32,7 @@ class SimpleDictReader:
     def __init__(self, fileobj, fieldnames):
         self.fieldnames = fieldnames
         self.contents = fileobj.read()
-        self.parser = sparse.SimpleParser(defaulttokenlist=[",", "\n"],whitespacechars="\r")
+        self.parser = sparse.SimpleParser(defaulttokenlist=[",", "\n"], whitespacechars="\r")
         self.parser.stringescaping = 0
         self.parser.quotechars = '"'
         self.tokens = self.parser.tokenize(self.contents)
@@ -143,6 +138,8 @@ class csvfile(base.TranslationStore):
     """This class represents a .csv file with various lines. 
     The default format contains three columns: comments, source, target"""
     UnitClass = csvunit
+    Mimetypes  = ['text/comma-separated-values', 'text/csv']
+    Extensions = ["csv"]
     def __init__(self, inputfile=None, fieldnames=None):
         base.TranslationStore.__init__(self, unitclass = self.UnitClass)
         self.units = []
@@ -189,4 +186,4 @@ if __name__ == '__main__':
     cf.parse(sys.stdin.read())
     sys.stdout.write(str(cf))
 
-	
\ No newline at end of file
+	
