@@ -25,6 +25,7 @@
 import re
 
 from translate.storage import base
+from translate.storage.placeables import lisa
 from translate.lang import data
 try:
     from lxml import etree
@@ -35,14 +36,16 @@ def getText(node):
     """joins together the text from all the text nodes in the nodelist and their children"""
     # node.xpath is very slow, so we only use it if there are children
     # TODO: consider rewriting by iterating over children
-    if node:    # The etree way of testing for children
-        return node.xpath("string()") # specific to lxml.etree
+    if node is not None:    # The etree way of testing for children
+        # Only non-ASCII strings are returned as unicode, so we have to force
+        # the ASCII-only ones to be unicode as well
+        return unicode(node.xpath("string()")) # specific to lxml.etree
     else:
         return data.forceunicode(node.text) or u""
         # if node.text is none, we want to return "" since the tag is there
 
 def _findAllMatches(text, re_obj):
-    """generate match objects for all @re_obj matches in @text."""
+    """generate match objects for all L{re_obj} matches in L{text}."""
     start = 0
     max = len(text)
     while start < max:
@@ -54,17 +57,21 @@ def _findAllMatches(text, re_obj):
 placeholders = ['(%[diouxXeEfFgGcrs])', r'(\\+.?)', '(%[0-9]$lx)', '(%[0-9]\$[a-z])', '(<.+?>)']
 re_placeholders = [re.compile(ph) for ph in placeholders]
 def _getPhMatches(text):
-    'return list of regexp matchobjects for with all place holders in the @text'
+    'return list of regexp matchobjects for with all place holders in the L{text}'
     matches = []
     for re_ph in re_placeholders:
         matches.extend(list(_findAllMatches(text, re_ph)))
 
     # sort them so they come sequentially
-    matches.sort(lambda a,b: cmp(a.start(),b.start()))
+    matches.sort(lambda a, b: cmp(a.start(), b.start()))
     return matches
 
 XML_NS = 'http://www.w3.org/XML/1998/namespace'
 
+def getXMLlang(node):
+    """Sets the xml:lang attribute on node"""
+    return node.get("{%s}lang" % XML_NS)
+
 def setXMLlang(node, lang):
     """Sets the xml:lang attribute on node"""
     node.set("{%s}lang" % XML_NS, lang)
@@ -76,9 +83,9 @@ def setXMLspace(node, value):
 def namespaced(namespace, name):
     """Returns name in Clark notation within the given namespace.
 
-    For example namespaced("source") in an XLIFF document might return
-        {urn:oasis:names:tc:xliff:document:1.1}source
-    This is needed throughout lxml.
+       For example namespaced("source") in an XLIFF document might return::
+           {urn:oasis:names:tc:xliff:document:1.1}source
+       This is needed throughout lxml.
     """
     if namespace:
         return "{%s}%s" % (namespace, name)
@@ -124,25 +131,77 @@ Provisional work is done to make several languages possible."""
     def namespaced(self, name):
         """Returns name in Clark notation.
 
-        For example namespaced("source") in an XLIFF document might return
-            {urn:oasis:names:tc:xliff:document:1.1}source
-        This is needed throughout lxml.
+           For example namespaced("source") in an XLIFF document might return::
+               {urn:oasis:names:tc:xliff:document:1.1}source
+           This is needed throughout lxml.
         """
         return namespaced(self.namespace, name)
 
-    def setsource(self, source, sourcelang='en'):
-        source = data.forceunicode(source)
+    def set_source_dom(self, dom_node):
         languageNodes = self.getlanguageNodes()
-        sourcelanguageNode = self.createlanguageNode(sourcelang, source, "source")
         if len(languageNodes) > 0:
-            self.xmlelement[0] = sourcelanguageNode
+            self.xmlelement[0] = dom_node
         else:
-            self.xmlelement.append(sourcelanguageNode)
+            self.xmlelement.append(dom_node)
+    
+    def get_source_dom(self):
+        return self.getlanguageNode(lang=None, index=0)
+    source_dom = property(get_source_dom, set_source_dom)
+
+    def _ensure_singular(cls, value):
+        if value is not None and len(value) > 1:
+            raise Exception("XLIFF cannot handle plurals by default")
+    _ensure_singular = classmethod(_ensure_singular)
+
+    def set_rich_source(self, value, sourcelang='en'):
+        self._ensure_singular(value)
+        sourcelanguageNode = self.createlanguageNode(sourcelang, u'', "source")        
+        self.source_dom = lisa.insert_into_dom(sourcelanguageNode, value[0])
+
+    def get_rich_source(self):
+        return [lisa.extract_chunks(self.source_dom)]
+    rich_source = property(get_rich_source, set_rich_source)
+
+    def setsource(self, text, sourcelang='en'):
+        text = data.forceunicode(text)
+        self.source_dom = self.createlanguageNode(sourcelang, text, "source")
 
     def getsource(self):
-        return self.getNodeText(self.getlanguageNode(lang=None, index=0))
+        return self.getNodeText(self.source_dom)
     source = property(getsource, setsource)
 
+    def set_target_dom(self, dom_node, append=False):
+        languageNodes = self.getlanguageNodes()
+        assert len(languageNodes) > 0
+        if dom_node is not None:
+            if append or len(languageNodes) == 1:
+                self.xmlelement.append(dom_node)
+            else:
+                self.xmlelement.insert(1, dom_node)
+        if not append and len(languageNodes) > 1:
+            self.xmlelement.remove(languageNodes[1])
+
+    def get_target_dom(self, lang=None):
+        if lang:
+            return self.getlanguageNode(lang=lang)
+        else:
+            return self.getlanguageNode(lang=None, index=1)
+    target_dom = property(get_target_dom)
+
+    def set_rich_target(self, value, lang='xx', append=False):
+        self._ensure_singular(value)
+        languageNode = None
+        if not value is None:
+            languageNode = self.createlanguageNode(lang, u'', "target")
+            lisa.insert_into_dom(languageNode, value[0])
+        self.set_target_dom(languageNode, append)
+
+    def get_rich_target(self, lang=None):
+        """retrieves the "target" text (second entry), or the entry in the 
+        specified language, if it exists"""
+        return [lisa.extract_chunks(self.get_target_dom(lang))]
+    rich_target = property(get_rich_target, set_rich_target)
+
     def settarget(self, text, lang='xx', append=False):
         #XXX: we really need the language - can't really be optional
         """Sets the "target" string (second language), or alternatively appends to the list"""
@@ -150,25 +209,15 @@ Provisional work is done to make several languages possible."""
         #Firstly deal with reinitialising to None or setting to identical string
         if self.gettarget() == text:
             return
-        languageNodes = self.getlanguageNodes()
-        assert len(languageNodes) > 0
+        languageNode = None
         if not text is None:
             languageNode = self.createlanguageNode(lang, text, "target")
-            if append or len(languageNodes) == 1:
-                self.xmlelement.append(languageNode)
-            else:
-                self.xmlelement.insert(1, languageNode)
-        if not append and len(languageNodes) > 1:
-            self.xmlelement.remove(languageNodes[1])
+        self.set_target_dom(languageNode, append)
 
     def gettarget(self, lang=None):
         """retrieves the "target" text (second entry), or the entry in the 
         specified language, if it exists"""
-        if lang:
-            node = self.getlanguageNode(lang=lang)
-        else:
-            node = self.getlanguageNode(lang=None, index=1)
-        return self.getNodeText(node)
+        return self.getNodeText(self.get_target_dom(lang))
     target = property(gettarget, settarget)
 
     def createlanguageNode(self, lang, text, purpose=None):
@@ -216,7 +265,7 @@ Provisional work is done to make several languages possible."""
         languageNodes = self.getlanguageNodes()
         if lang:
             for set in languageNodes:
-                if set.get("{%s}lang" % XML_NS) == lang:
+                if getXMLlang(set) == lang:
                     return set
         else:#have to use index
             if index >= len(languageNodes):
@@ -240,6 +289,15 @@ Provisional work is done to make several languages possible."""
     def __str__(self):
         return etree.tostring(self.xmlelement, pretty_print=True, encoding='utf-8')
 
+    def _set_property(self, name, value):
+        self.xmlelement.attrib[name] = value
+
+    xid = property(lambda self:        self.xmlelement.attrib[self.namespaced('xid')],
+                   lambda self, value: self._set_property(self.namespaced('xid'), value))
+
+    rid = property(lambda self:        self.xmlelement.attrib[self.namespaced('rid')],
+                   lambda self, value: self._set_property(self.namespaced('rid'), value))
+
     def createfromxmlElement(cls, element):
         term = cls(None, empty=True)
         term.xmlelement = element
@@ -278,9 +336,9 @@ class LISAfile(base.TranslationStore):
     def namespaced(self, name):
         """Returns name in Clark notation.
 
-        For example namespaced("source") in an XLIFF document might return
-            {urn:oasis:names:tc:xliff:document:1.1}source
-        This is needed throughout lxml.
+           For example namespaced("source") in an XLIFF document might return::
+               {urn:oasis:names:tc:xliff:document:1.1}source
+           This is needed throughout lxml.
         """
         return namespaced(self.namespace, name)
 
@@ -289,14 +347,6 @@ class LISAfile(base.TranslationStore):
         self.namespace = self.document.getroot().nsmap.get(None, None)
         self.body = self.document.find('//%s' % self.namespaced(self.bodyNode))
 
-    def setsourcelanguage(self, sourcelanguage):
-        """Sets the source language for this store"""
-        self.sourcelanguage = sourcelanguage
-
-    def settargetlanguage(self, targetlanguage):
-        """Sets the target language for this store"""
-        self.targetlanguage = targetlanguage
-
     def addsourceunit(self, source):
         #TODO: miskien moet hierdie eerder addsourcestring of iets genoem word?
         """Adds and returns a new unit with the given string as first entry."""
@@ -304,10 +354,11 @@ class LISAfile(base.TranslationStore):
         self.addunit(newunit)
         return newunit
 
-    def addunit(self, unit):
+    def addunit(self, unit, new=True):
         unit.namespace = self.namespace
-        self.body.append(unit.xmlelement)
-        self.units.append(unit)
+        super(LISAfile, self).addunit(unit)
+        if new:
+            self.body.append(unit.xmlelement)
 
     def __str__(self):
         """Converts to a string containing the file's XML"""
@@ -330,6 +381,5 @@ class LISAfile(base.TranslationStore):
             return
         for entry in termEntries:
             term = self.UnitClass.createfromxmlElement(entry)
-            term.namespace = self.namespace
-            self.units.append(term)
+            self.addunit(term, new=False)
 
