@@ -41,6 +41,19 @@ def parseheaderstring(input):
         headervalues[key] = value.strip()
     return headervalues
 
+def tzstring():
+    """Returns the timezone as a string in the format [+-]0000, eg +0200."""
+    if time.daylight:
+        tzoffset = time.altzone
+    else:
+        tzoffset = time.timezone
+
+    hours, minutes = time.gmtime(abs(tzoffset))[3:5]
+    if tzoffset > 0:
+        hours *= -1
+    tz = str("%+d" % hours).zfill(3) + str(minutes).zfill(2)
+    return tz
+
 def update(existing, add=False, **kwargs):
     """Update an existing header dictionary with the values in kwargs, adding new values 
     only if add is true. 
@@ -74,7 +87,7 @@ def update(existing, add=False, **kwargs):
     return headerargs
 
 
-class poheader:
+class poheader(object):
     """This class implements functionality for manipulation of po file headers.
     This class is a mix-in class and useless on its own. It must be used from all
     classes which represent a po file"""
@@ -88,6 +101,7 @@ class poheader:
         "PO-Revision-Date",
         "Last-Translator",
         "Language-Team",
+        "Language",
         "MIME-Version",
         "Content-Type",
         "Content-Transfer-Encoding",
@@ -95,19 +109,6 @@ class poheader:
         "X-Generator",
         ]
 
-    def tzstring(self):
-        """Returns the timezone as a string in the format [+-]0000, eg +0200."""
-        if time.daylight:
-            tzoffset = time.altzone
-        else:
-            tzoffset = time.timezone
-
-        hours, minutes = time.gmtime(abs(tzoffset))[3:5]
-        if tzoffset > 0:
-            hours *= -1
-        tz = str("%+d" % hours).zfill(3) + str(minutes).zfill(2)
-        return tz
-
 
     def makeheaderdict(self, charset="CHARSET", encoding="ENCODING", project_id_version=None, pot_creation_date=None, po_revision_date=None, last_translator=None, language_team=None, mime_version=None, plural_forms=None, report_msgid_bugs_to=None, **kwargs):
         """create a header for the given filename. arguments are specially handled, kwargs added as key: value
@@ -120,17 +121,17 @@ class poheader:
         if project_id_version is None:
             project_id_version = "PACKAGE VERSION"
         if pot_creation_date is None or pot_creation_date == True:
-            pot_creation_date = time.strftime("%Y-%m-%d %H:%M") + self.tzstring()
+            pot_creation_date = time.strftime("%Y-%m-%d %H:%M") + tzstring()
         if isinstance(pot_creation_date, time.struct_time):
-            pot_creation_date = time.strftime("%Y-%m-%d %H:%M", pot_creation_date) + self.tzstring()
+            pot_creation_date = time.strftime("%Y-%m-%d %H:%M", pot_creation_date) + tzstring()
         if po_revision_date is None:
             po_revision_date = "YEAR-MO-DA HO:MI+ZONE"
         elif po_revision_date == False:
             po_revision_date = pot_creation_date
         elif po_revision_date == True:
-            po_revision_date = time.strftime("%Y-%m-%d %H:%M") + self.tzstring()
+            po_revision_date = time.strftime("%Y-%m-%d %H:%M") + tzstring()
         if isinstance(po_revision_date, time.struct_time):
-            po_revision_date = time.strftime("%Y-%m-%d %H:%M", po_revision_date) + self.tzstring()
+            po_revision_date = time.strftime("%Y-%m-%d %H:%M", po_revision_date) + tzstring()
         if last_translator is None:
             last_translator = "FULL NAME <EMAIL@ADDRESS>"
         if language_team is None:
@@ -187,6 +188,11 @@ class poheader:
                 self.units.insert(0, header)
         else:
             headeritems = update(self.parseheader(), add, **kwargs)
+            keys = headeritems.keys()
+            if not "Content-Type" in keys or "charset=CHARSET" in headeritems["Content-Type"]:
+                headeritems["Content-Type"] = "text/plain; charset=UTF-8"
+            if not "Content-Transfer-Encoding" in keys or "ENCODING" in headeritems["Content-Transfer-Encoding"]:
+                headeritems["Content-Transfer-Encoding"] = "8bit"
             headerString = ""
             for key, value in headeritems.items():
                 headerString += "%s: %s\n" % (key, value)
@@ -216,7 +222,15 @@ class poheader:
         """update the Plural-Form PO header"""
         if isinstance(nplurals, basestring):
             nplurals = int(nplurals)
-        self.updateheader( Plural_Forms = "nplurals=%d; plural=%s;" % (nplurals, plural) )
+        self.updateheader(add=True, Plural_Forms = "nplurals=%d; plural=%s;" % (nplurals, plural) )
+
+    def gettargetlanguage(self):
+        header = self.parseheader()
+        return header.get('Language')
+
+    def settargetlanguage(self, lang):
+        if isinstance(lang, basestr) and len(lang) > 1:
+            self.updateheader(add=True, Language=lang)
 
     def mergeheaders(self, otherstore):
         """Merges another header with this header.
@@ -228,13 +242,17 @@ class poheader:
         """
 
         newvalues = otherstore.parseheader()
-        self.updateheader(
-            Project_Id_Version = newvalues['Project-Id-Version'],
-            PO_Revision_Date   = newvalues['PO-Revision-Date'],
-            Last_Translator    = newvalues['Last-Translator'],
-            Language_Team      = newvalues['Language-Team'],
-            Plural_Forms       = newvalues['Plural-Forms']
-        )
+        retain = {
+                "Project_Id_Version": newvalues['Project-Id-Version'],
+                "PO_Revision_Date"  : newvalues['PO-Revision-Date'],
+                "Last_Translator"   : newvalues['Last-Translator'],
+                "Language_Team"     : newvalues['Language-Team'],
+        }
+        # not necessarily there:
+        plurals = newvalues.get('Plural-Forms', None)
+        if plurals:
+            retain['Plural-Forms'] = plurals
+        self.updateheader(**retain)
 
     def updatecontributor(self, name, email=None):
         """Add contribution comments
