@@ -27,7 +27,7 @@ from translate.misc.multistring import multistring
 from translate.misc import quote
 from translate.misc import textwrap
 from translate.lang import data
-from translate.storage import pocommon
+from translate.storage import pocommon, base
 import re
 
 lsep = "\n#: "
@@ -103,16 +103,11 @@ def extractpoline(line):
      
     @param line: a quoted line from a po file (msgid or msgstr)
     """
-    extracted = quote.extractwithoutquotes(line,'"','"','\\',includeescapes=unescapehandler)[0]
+    extracted = quote.extractwithoutquotes(line, '"', '"', '\\', includeescapes=unescapehandler)[0]
     return extracted
 
-def unquotefrompo(postr, joinwithlinebreak=False):
-    if joinwithlinebreak:
-        joiner = u"\n"
-        if postr and postr[0] == '""': postr = postr[1:]
-    else:
-        joiner = u""
-    return joiner.join([extractpoline(line) for line in postr])
+def unquotefrompo(postr):
+    return u"".join([extractpoline(line) for line in postr])
 
 def encodingToUse(encoding):
     """Tests whether the given encoding is known in the python runtime, or returns utf-8.
@@ -140,6 +135,14 @@ From the GNU gettext manual:
      msgstr TRANSLATED-STRING
 """
 
+def extractstr(string):
+    left = string.find('"')
+    right = string.rfind('"')
+    if right > -1:
+        return string[left:right+1]
+    else:
+        return string[left:] + '"'
+
 class pounit(pocommon.pounit):
     # othercomments = []      #   # this is another comment
     # automaticcomments = []  #   #. comment extracted from the source code
@@ -368,7 +371,7 @@ class pounit(pocommon.pounit):
                 for item in list2:
                     splitlist2.extend(item.split()[1:])
                     prefix = item.split()[0]
-                list1.extend(["%s %s%s" % (prefix,item,lineend) for item in splitlist2 if not item in splitlist1])
+                list1.extend(["%s %s%s" % (prefix, item, lineend) for item in splitlist2 if not item in splitlist1])
             else:
                 #Normal merge, but conform to list1 newline style
                 if list1 != list2:
@@ -509,10 +512,7 @@ class pounit(pocommon.pounit):
         """returns whether this pounit contains plural strings..."""
         return len(self.msgid_plural) > 0
 
-    def parse(self, src):
-        if isinstance(src, str):
-            # This has not been decoded yet, so we need to make a plan
-            src = src.decode(self._encoding)
+    def parselines(self, lines):
         inmsgctxt = 0
         inmsgid = 0
         inmsgid_comment = 0
@@ -520,7 +520,7 @@ class pounit(pocommon.pounit):
         inmsgstr = 0
         msgstr_pluralid = None
         linesprocessed = 0
-        for line in src.split("\n"):
+        for line in lines:
             line = line + "\n"
             linesprocessed += 1
             if len(line) == 0:
@@ -571,7 +571,7 @@ class pounit(pocommon.pounit):
                     msgstr_pluralid = int(line[len('msgstr['):line.find(']')].strip())
                 else:
                     msgstr_pluralid = None
-            extracted = quote.extractstr(line)
+            extracted = extractstr(line)
             if not extracted is None:
                 if inmsgctxt:
                     self.msgctxt.append(extracted)
@@ -613,6 +613,12 @@ class pounit(pocommon.pounit):
                 self._encoding = encodingToUse(charset.group(1))
         return linesprocessed
 
+    def parse(self, src):
+        if isinstance(src, str):
+            # This has not been decoded yet, so we need to make a plan
+            src = src.decode(self._encoding)
+        return self.parselines(src.split("\n"))
+
     def _getmsgpartstr(self, partname, partlines, partcomments=""):
         if isinstance(partlines, dict):
             partkeys = partlines.keys()
@@ -780,8 +786,8 @@ class pofile(pocommon.pofile):
         if not header or header.isblank():
             return
         charsetline = None
-        headerstr = unquotefrompo(header.msgstr, True)
-        for line in headerstr.split("\\n"):
+        headerstr = unquotefrompo(header.msgstr)
+        for line in headerstr.split("\n"):
             if not ":" in line: continue
             key, value = line.strip().split(":", 1)
             if key.strip() != "Content-Type": continue
@@ -803,42 +809,50 @@ class pofile(pocommon.pofile):
 
     def parse(self, input):
         """parses the given file or file source string"""
-        if hasattr(input, 'name'):
-            self.filename = input.name
-        elif not getattr(self, 'filename', ''):
-            self.filename = ''
-        if hasattr(input, "read"):
-            posrc = input.read()
-            input.close()
-            input = posrc
-        # TODO: change this to a proper parser that doesn't do line-by-line madness
-        lines = input.split("\n")
-        start = 0
-        end = 0
-        # make only the first one the header
-        linesprocessed = 0
-        while end <= len(lines):
-            if (end == len(lines)) or (not lines[end].strip()):  # end of lines or blank line
-                newpe = self.UnitClass(encoding=self._encoding)
-                linesprocessed = newpe.parse("\n".join(lines[start:end]))
-                start += linesprocessed
-                # TODO: find a better way of working out if we actually read anything
-                if linesprocessed >= 1 and newpe._getoutput():
-                    self.units.append(newpe)
-                    if newpe.isheader():
-                        if "Content-Type" in self.parseheader():
-                            self._encoding = newpe._encoding
-                        # now that we know the encoding, decode the whole file
-                        if self._encoding is not None and self._encoding.lower() != 'charset':
+        try:
+            if hasattr(input, 'name'):
+                self.filename = input.name
+            elif not getattr(self, 'filename', ''):
+                self.filename = ''
+            if hasattr(input, "read"):
+                posrc = input.read()
+                input.close()
+                input = posrc
+            # TODO: change this to a proper parser that doesn't do line-by-line madness
+            lines = input.split("\n")
+            start = 0
+            end = 0
+            # make only the first one the header
+            linesprocessed = 0
+            is_decoded = False
+            while end <= len(lines):
+                if (end == len(lines)) or (not lines[end].strip()):  # end of lines or blank line
+                    newpe = self.UnitClass(encoding=self._encoding)
+                    unit_lines = lines[start:end]
+                    # We need to work carefully if we haven't decoded properly yet.
+                    # So let's solve this temporarily until we actually get the
+                    # encoding from the header.
+                    if not is_decoded:
+                        unit_lines = [line.decode('ascii', 'ignore') for line in unit_lines]
+                    linesprocessed = newpe.parselines(unit_lines)
+                    start += linesprocessed
+                    # TODO: find a better way of working out if we actually read anything
+                    if linesprocessed >= 1 and newpe._getoutput():
+                        self.units.append(newpe)
+                        if not is_decoded:
+                            if newpe.isheader(): # If there is a header...
+                                if "Content-Type" in self.parseheader(): # and a Content-Type...
+                                    if self._encoding.lower() != 'charset': # with a valid charset...
+                                        self._encoding = newpe._encoding # then change the encoding
+                                        # otherwise we'll decode using UTF-8
                             lines = self.decode(lines)
-                    if self._encoding is None: #still have not found an encoding, let's assume UTF-8
-                        #TODO: This might be dead code
-                        self._encoding = 'utf-8'
-                        lines = self.decode(lines)
-                        self.units = []
-                        start = 0
-                        end = 0
-            end = end+1
+                            self.units = []
+                            start = 0
+                            end = 0
+                            is_decoded = True
+                end = end+1
+        except Exception, e:
+            raise base.ParseError()
 
     def removeduplicates(self, duplicatestyle="merge"):
         """make sure each msgid is unique ; merge comments etc from duplicates into original"""
