@@ -5,6 +5,7 @@ from translate.storage import po
 from translate.storage import test_base
 from translate.misc import wStringIO
 from translate.misc.multistring import multistring
+from translate.storage.placeables import X, G
 from py.test import raises
 
 def test_roundtrip_quoting():
@@ -41,8 +42,9 @@ class TestPOUnit(test_base.TestTranslationUnit):
     def test_adding_empty_note(self):
         unit = self.UnitClass("bla")
         assert not '#' in str(unit)
-        unit.addnote("")
-        assert not '#' in str(unit)
+        for empty_string in [ "", " ", "\t", "\n" ]:
+            unit.addnote(empty_string)
+            assert not '#' in str(unit)
 
     def test_markreview(self):
         """Tests if we can mark the unit to need review."""
@@ -129,6 +131,16 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.source = u"Goeiemôre"
         assert not unit.isheader()
 
+#     def test_rich_source(self):
+#         unit = self.unit
+#         unit.rich_source = [['a', X('42'), 'c']]
+#         assert unit.rich_source == [[u'a\ufffcc']]
+
+#     def test_rich_target(self):
+#         unit = self.unit
+#         unit.rich_target = [['a', G('42', ['b']), 'c']]
+#         assert unit.rich_target == [['abc']]
+
 class TestPOFile(test_base.TestTranslationStore):
     StoreClass = po.pofile
     def poparse(self, posource):
@@ -146,10 +158,10 @@ class TestPOFile(test_base.TestTranslationStore):
         oldpofile = self.poparse(oldmessage)
         oldunit = oldpofile.units[0]
         if newmessage:
-          newpofile = self.poparse(newmessage)
-          newunit = newpofile.units[0]
+            newpofile = self.poparse(newmessage)
+            newunit = newpofile.units[0]
         else:
-          newunit = oldpofile.UnitClass()
+            newunit = oldpofile.UnitClass()
         oldunit.merge(newunit, authoritative=authoritative)
         print oldunit
         return str(oldunit)
