@@ -20,6 +20,73 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """Manage the Wordfast Translation Memory format
+
+   Wordfast TM format is the Translation Memory format used by the 
+   U{Wordfast<http://www.wordfast.net/>} computer aided translation tool.
+
+   It is a bilingual base class derived format with L{WordfastTMFile}
+   and L{WordfastUnit} providing file and unit level access.
+
+   Wordfast tools
+   ==============
+   Wordfast is a computer aided translation tool.  It is an application
+   built on top of Microsoft Word and is implemented as a rather 
+   sophisticated set of macros.  Understanding that helps us understand
+   many of the seemingly strange choices around this format including:
+   encoding, escaping and file naming.
+
+   Implementation
+   ==============
+   The implementation covers the full requirements of a Wordfast TM file.
+   The files are simple Tab Seperated Value (TSV) files that can be read 
+   by Microsoft Excel and other spreadsheet programs.  They use the .txt 
+   extension which does make it more difficult to automatically identify 
+   such files.
+
+   The dialect of the TSV files is specified by L{WordfastDialect}.
+
+   Encoding
+   --------
+   The files are UTF-16 or ISO-8859-1 (Latin1) encoded.  These choices
+   are most likely because Microsoft Word is the base editing tool for
+   Wordfast.
+
+   The format is tab seperated so We are able to detect UTF-16 vs Latin1 
+   by searching for the occurance of a UTF-16 tab character and then
+   continuing with the parsing.
+
+   Timestamps
+   ----------
+   L{WordfastTime} allows for the correct management of the Wordfast
+   YYYYMMDD~HHMMSS timestamps.  However, timestamps on individual units are 
+   not updated when edited.
+
+   Header
+   ------
+   L{WordfastHeader} provides header management support.  The header 
+   functionality is fully implemented through observing the behaviour of the
+   files in real use cases, input from the Wordfast programmers and 
+   public documentation.
+
+   Escaping
+   --------
+   Wordfast TM implements a form of escaping that covers two aspects:
+     1. Placeable: bold, formating, etc.  These are left as is and ignored.
+        It is up to the editor and future placeable implementation to manage
+        these.
+     2. Escapes: items that may confuse Excel or translators are 
+        escaped as &'XX;. These are fully implemented and are converted to
+        and from Unicode.  By observing behaviour and reading documentation
+        we where able to observe all possible escapes. Unfortunately the
+        escaping differs slightly between Windows and Mac version.  This
+        might cause errors in future.
+   Functions allow for L{conversion to Unicode<_wf_to_char>} and L{back to 
+   Wordfast escapes<_char_to_wf>}.
+
+   Extended Attributes
+   -------------------
+   The last 4 columns allow users to define and manage extended attributes.
+   These are left as is and are not directly managed byour implemenation.
 """
 
 import csv
@@ -92,11 +159,13 @@ WF_ESCAPE_MAP = (
 """Mapping of Wordfast &'XX; escapes to correct Unicode characters"""
 
 TAB_UTF16 = "\x00\x09"
+"""The tab \\t character as it would appear in UTF-16 encoding"""
 
 def _char_to_wf(string):
     """Char -> Wordfast &'XX; escapes
     
-    @note: Full roundtripping is not possible because of the escaping of \n and \t"""
+       Full roundtripping is not possible because of the escaping of NEWLINE \\n 
+       and TAB \\t"""
     # FIXME there is no platform check to ensure that we use Mac encodings when running on a Mac
     if string:
         for code, char in WF_ESCAPE_MAP:
@@ -112,6 +181,17 @@ def _wf_to_char(string):
         string = string.replace("\\n", "\n").replace("\\t", "\t")
     return string
 
+class WordfastDialect(csv.Dialect):
+    """Describe the properties of a Wordfast generated TAB-delimited file."""
+    delimiter = "\t"
+    lineterminator = "\r\n"
+    quoting = csv.QUOTE_NONE
+    # We need to define the following 3 items for csv in Python < 2.5
+    doublequote = False
+    skipinitialspace = False
+    escapechar = ''
+csv.register_dialect("wordfast", WordfastDialect)
+
 class WordfastTime(object):
     """Manages time stamps in the Wordfast format of YYYYMMDD~hhmmss"""
     def __init__(self, newtime=None):
@@ -134,7 +214,7 @@ class WordfastTime(object):
         """Set the time_sturct object using a Wordfast time formated string
 
         @param timestring: A Wordfast time string (YYYMMDD~hhmmss)
-        @param type: String
+        @type timestring: String
         """
         self._time = time.strptime(timestring, WF_TIMEFORMAT)
     timestring = property(get_timestring, set_timestring)
@@ -265,6 +345,9 @@ class WordfastUnit(base.TranslationUnit):
 
 class WordfastTMFile(base.TranslationStore):
     """A Wordfast translation memory file"""
+    Name = "Wordfast TM file"
+    Mimetypes  = ["application/x-wordfast"]
+    Extensions = ["txt"]
     def __init__(self, inputfile=None, unitclass=WordfastUnit):
         """construct a Wordfast TM, optionally reading in from inputfile."""
         self.UnitClass = unitclass
@@ -293,9 +376,9 @@ class WordfastTMFile(base.TranslationStore):
             input = input.decode(self._encoding).encode('utf-8')
         except:
             raise ValueError("Wordfast files are either UTF-16 (UCS2) or ISO-8859-1 encoded")
-        for header in csv.DictReader(input.split("\n")[:1], fieldnames=WF_FIELDNAMES_HEADER, dialect="excel-tab"):
+        for header in csv.DictReader(input.split("\n")[:1], fieldnames=WF_FIELDNAMES_HEADER, dialect="wordfast"):
             self.header = WordfastHeader(header)
-        lines = csv.DictReader(input.split("\n")[1:], fieldnames=WF_FIELDNAMES, dialect="excel-tab")
+        lines = csv.DictReader(input.split("\n")[1:], fieldnames=WF_FIELDNAMES, dialect="wordfast")
         for line in lines:
             newunit = WordfastUnit()
             newunit.dict = line
@@ -304,7 +387,7 @@ class WordfastTMFile(base.TranslationStore):
     def __str__(self):
         output = csv.StringIO()
         header_output = csv.StringIO()
-        writer = csv.DictWriter(output, fieldnames=WF_FIELDNAMES, dialect="excel-tab")
+        writer = csv.DictWriter(output, fieldnames=WF_FIELDNAMES, dialect="wordfast")
         unit_count = 0
         for unit in self.units:
             if unit.istranslated():
@@ -314,7 +397,7 @@ class WordfastTMFile(base.TranslationStore):
             return ""
         output.reset()
         self.header.tucount = unit_count
-        outheader = csv.DictWriter(header_output, fieldnames=WF_FIELDNAMES_HEADER, dialect="excel-tab")
+        outheader = csv.DictWriter(header_output, fieldnames=WF_FIELDNAMES_HEADER, dialect="wordfast")
         outheader.writerow(self.header.header)
         header_output.reset()
         decoded = "".join(header_output.readlines() + output.readlines()).decode('utf-8')
