@@ -69,7 +69,7 @@ class xliffunit(lisa.LISAunit):
             nodes.extend(sources[- (sourcesl - targetsl):])
         return nodes
 
-    def addalttrans(self, txt, origin=None, lang=None):
+    def addalttrans(self, txt, origin=None, lang=None, sourcetxt=None, matchquality=None):
         """Adds an alt-trans tag and alt-trans components to the unit.
         
         @type txt: String
@@ -83,6 +83,13 @@ class xliffunit(lisa.LISAunit):
         alttrans = etree.SubElement(self.xmlelement, self.namespaced("alt-trans"))
         alttarget = etree.SubElement(alttrans, self.namespaced("target"))
         alttarget.text = txt
+        if sourcetxt:
+            if isinstance(sourcetxt, str):
+                sourcetxt = sourcetxt.decode("utf-8")
+            altsource = etree.SubElement(alttrans, self.namespaced("source"))
+            altsource.text = sourcetxt
+        if matchquality:
+            alttrans.set("match-quality", matchquality)
         if origin:
             alttrans.set("origin", origin)
         if lang:
@@ -174,7 +181,7 @@ class xliffunit(lisa.LISAunit):
         """Mark this unit as approved."""
         if value:
             self.xmlelement.set("approved", "yes")
-        else:
+        elif self.isapproved():
             self.xmlelement.set("approved", "no")
 
     def isreview(self):
@@ -236,7 +243,7 @@ class xliffunit(lisa.LISAunit):
 
     def marktranslated(self):
         targetnode = self.getlanguageNode(lang=None, index=1)
-        if not targetnode:
+        if targetnode is None:
             return
         if self.isfuzzy() and "state-qualifier" in targetnode.attrib:
             #TODO: consider
@@ -259,7 +266,14 @@ class xliffunit(lisa.LISAunit):
         """Add the context group to the trans-unit with contexts a list with
         (type, text) tuples describing each context."""
         assert contexts
-        group = etree.SubElement(self.xmlelement, self.namespaced("context-group"))
+        group = etree.Element(self.namespaced("context-group"))
+        # context-group tags must appear at the start within <group>
+        # tags. Otherwise it must be appended to the end of a group
+        # of tags.
+        if self.xmlelement.tag == self.namespaced("group"):
+            self.xmlelement.insert(0, group)
+        else:
+            self.xmlelement.append(group)
         group.set("name", name)
         if purpose:
             group.set("purpose", purpose)
@@ -287,13 +301,15 @@ class xliffunit(lisa.LISAunit):
         """returns the restype attribute in the trans-unit tag"""
         return self.xmlelement.get("restype")
 
-    def merge(self, otherunit, overwrite=False, comments=True):
+    def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
         #TODO: consider other attributes like "approved"
         super(xliffunit, self).merge(otherunit, overwrite, comments)
         if self.target:
             self.marktranslated()
-        if otherunit.isfuzzy():
-            self.markfuzzy()
+            if otherunit.isfuzzy():
+                self.markfuzzy()
+            elif otherunit.source == self.source:
+                self.markfuzzy(False)
 
     def correctorigin(self, node, origin):
         """Check against node tag's origin (e.g note or alt-trans)"""
@@ -309,6 +325,9 @@ class xliffunit(lisa.LISAunit):
 class xlifffile(lisa.LISAfile):
     """Class representing a XLIFF file store."""
     UnitClass = xliffunit
+    Name = "XLIFF file"
+    Mimetypes  = ["application/x-xliff", "application/x-xliff+xml"]
+    Extensions = ["xlf", "xliff"]
     rootNode = "xliff"
     bodyNode = "body"
     XMLskeleton = '''<?xml version="1.0" ?>
@@ -392,6 +411,20 @@ class xlifffile(lisa.LISAfile):
                 return self.getdatatype(filenames[0])
         return ""
 
+    def getdate(self, filename=None):
+        """Returns the date attribute for the file. If no filename is given,
+        the date of the first file is given. If the date attribute is not
+        specified, None is returned."""
+        if filename:
+            node = self.getfilenode(filename)
+            if not node is None:
+                return node.get("date")
+        else:
+            filenames = self.getfilenames()
+            if len(filenames) > 0 and filenames[0] != "NoName":
+                return self.getdate(filenames[0])
+        return None
+
     def removedefaultfile(self):
         """We want to remove the default file-tag as soon as possible if we 
         know if still present and empty."""
