@@ -156,11 +156,13 @@ Review Messages, Review Source Words"
             self.totals[key] += stats[key]
 
     def handlefile(self, filename):
-        stats = calcstats(filename)
-        if stats:
+        try:
+            stats = calcstats(filename)
             self.updatetotals(stats)
             summarize(filename, stats, self.CSVstyle)
             self.filecount += 1
+        except: # This happens if we have a broken file.
+            print >> sys.stderr, sys.exc_info()[1]
 
     def handlefiles(self, dirname, filenames):
         for filename in filenames:
@@ -172,7 +174,7 @@ Review Messages, Review Source Words"
 
     def handledir(self, dirname):
         path, name = os.path.split(dirname)
-        if name in ["CVS", ".svn", "_darcs"]:
+        if name in ["CVS", ".svn", "_darcs", ".git", ".hg", ".bzr"]:
             return
         entries = os.listdir(dirname)
         self.handlefiles(dirname, entries)
