@@ -26,10 +26,16 @@ usage instructions
 """
 
 from translate.storage import factory
+from translate.misc.rich import map_rich, only_strings
 import os
 import re
 import md5
 
+def add_prefix(prefix, strings):
+    for string in strings:
+        string.insert(0, prefix)
+    return strings
+
 class podebug:
     def __init__(self, format=None, rewritestyle=None, hash=None, ignoreoption=None):
         if format is None:
@@ -40,6 +46,10 @@ class podebug:
         self.ignorefunc = getattr(self, "ignore_%s" % ignoreoption, None)
         self.hash = hash
 
+    def rewritelist(cls):
+        return [rewrite.replace("rewrite_", "") for rewrite in dir(cls) if rewrite.startswith("rewrite_")]
+    rewritelist = classmethod(rewritelist)
+
     def rewrite_xxx(self, string):
         return "xxx%sxxx" % string
 
@@ -49,6 +59,54 @@ class podebug:
     def rewrite_blank(self, string):
         return ""
 
+    def rewrite_chef(self, string):
+        """Rewrite using Mock Swedish as made famous by Monty Python"""
+        # From Dive into Python which itself got it elsewhere http://www.renderx.com/demos/examples/diveintopython.pdf
+        subs = (
+               (r'a([nu])', r'u\1'),
+               (r'A([nu])', r'U\1'),
+               (r'a\B', r'e'),
+               (r'A\B', r'E'),
+               (r'en\b', r'ee'),
+               (r'\Bew', r'oo'),
+               (r'\Be\b', r'e-a'),
+               (r'\be', r'i'),
+               (r'\bE', r'I'),
+               (r'\Bf', r'ff'),
+               (r'\Bir', r'ur'),
+               (r'(\w*?)i(\w*?)$', r'\1ee\2'),
+               (r'\bow', r'oo'),
+               (r'\bo', r'oo'),
+               (r'\bO', r'Oo'),
+               (r'the', r'zee'),
+               (r'The', r'Zee'),
+               (r'th\b', r't'),
+               (r'\Btion', r'shun'),
+               (r'\Bu', r'oo'),
+               (r'\BU', r'Oo'),
+               (r'v', r'f'),
+               (r'V', r'F'),
+               (r'w', r'w'),
+               (r'W', r'W'),
+               (r'([a-z])[.]', r'\1. Bork Bork Bork!'))
+        for a, b in subs:
+            string = re.sub(a, b, string)
+        return string
+
+    REWRITE_UNICODE_MAP = u"ȦƁƇḒḖƑƓĦĪĴĶĿḾȠǾƤɊŘŞŦŬṼẆẊẎẐ" + u"[\\]^_`" + u"ȧƀƈḓḗƒɠħīĵķŀḿƞǿƥɋřşŧŭṽẇẋẏẑ"
+    def rewrite_unicode(self, string):
+        """Convert to Unicode characters that look like the source string"""
+        def transpose(char):
+            loc = ord(char)-65
+            if loc < 0 or loc > 56:
+                return char
+            return self.REWRITE_UNICODE_MAP[loc]
+        return "".join(map(transpose, string))
+
+    def ignorelist(cls):
+        return [rewrite.replace("ignore_", "") for rewrite in dir(cls) if rewrite.startswith("ignore_")]
+    ignorelist = classmethod(ignorelist)
+
     def ignore_openoffice(self, locations):
         for location in locations:
             if location.startswith("Common.xcu#..Common.View.Localisation"):
@@ -83,16 +141,10 @@ class podebug:
                 hashable = unit.source
             prefix = md5.new(hashable).hexdigest()[:self.hash] + " "
         if self.rewritefunc:
-            unit.target = self.rewritefunc(unit.source)
+            unit.rich_target = map_rich(only_strings(self.rewritefunc), unit.rich_source)
         elif not unit.istranslated():
-            unit.target = unit.source
-        if unit.hasplural():
-            strings = unit.target.strings
-            for i, string in enumerate(strings):
-                strings[i] = prefix + string
-            unit.target = strings
-        else:
-            unit.target = prefix + unit.target
+            unit.rich_target = unit.rich_source
+        unit.rich_target = add_prefix(prefix, unit.rich_target)
         return unit
 
     def convertstore(self, store):
@@ -162,12 +214,10 @@ def main():
     parser = convert.ConvertOptionParser(formats, usepots=True, description=__doc__)
     # TODO: add documentation on format strings...
     parser.add_option("-f", "--format", dest="format", default="[%s] ", help="specify format string")
-    rewritestylelist = ["xxx", "en", "blank"]
     parser.add_option("", "--rewrite", dest="rewritestyle", 
-        type="choice", choices=rewritestylelist, metavar="STYLE", help="the translation rewrite style: %s" % ", ".join(rewritestylelist))
-    ignoreoptionlist = ["openoffice", "mozilla"]
+        type="choice", choices=podebug.rewritelist(), metavar="STYLE", help="the translation rewrite style: %s" % ", ".join(podebug.rewritelist()))
     parser.add_option("", "--ignore", dest="ignoreoption", 
-        type="choice", choices=ignoreoptionlist, metavar="APPLICATION", help="apply tagging ignore rules for the given application: %s" % ", ".join(ignoreoptionlist))
+        type="choice", choices=podebug.ignorelist(), metavar="APPLICATION", help="apply tagging ignore rules for the given application: %s" % ", ".join(podebug.ignorelist()))
     parser.add_option("", "--hash", dest="hash", metavar="LENGTH", type="int", help="add an md5 hash to translations")
     parser.passthrough.append("format")
     parser.passthrough.append("rewritestyle")
