@@ -66,6 +66,8 @@ class GrepFilter:
         self.includeheader = includeheader
 
     def matches(self, teststr):
+        if teststr is None:
+            return False
         teststr = data.normalize(teststr)
         if self.ignorecase:
             teststr = teststr.lower()
@@ -111,6 +113,8 @@ class GrepFilter:
     def filterfile(self, thefile):
         """runs filters on a translation file object"""
         thenewfile = type(thefile)()
+        thenewfile.setsourcelanguage(thefile.sourcelanguage)
+        thenewfile.settargetlanguage(thefile.targetlanguage)
         for unit in thefile.units:
             if self.filterunit(unit):
                 thenewfile.addunit(unit)
